@@ -8,8 +8,6 @@ public class ClassType extends ObjectType implements AttrContainer {
   public static final int minor_version = 3;
   public static final int major_version = 45;
 
-  static java.util.Hashtable classTable;
-
   /** Find a ClassType with the given name, or create a new one.
    * Use this for "library classes", where you need the field/method types,
    * but not one where you are about to generate code for.
@@ -17,15 +15,7 @@ public class ClassType extends ObjectType implements AttrContainer {
    */
   public static ClassType make(String name)
   {
-    if (classTable == null)
-      classTable = new java.util.Hashtable();
-    ClassType cl = (ClassType) classTable.get(name);
-    if (cl == null)
-      {
-	cl = new ClassType(name);
-	classTable.put(name, cl);
-      }
-    return cl;
+    return (ClassType) Type.getType(name);
   }
 
   public static ClassType make (String name, ClassType superClass)
@@ -74,9 +64,8 @@ public class ClassType extends ObjectType implements AttrContainer {
   public final int getModifiers()
   {
     if (access_flags == 0 && getReflectClass() != null)
-      return reflectClass.getModifiers();
-    else
-      return access_flags;
+      access_flags = reflectClass.getModifiers();
+    return access_flags;
   }
 
   /** Set the modifiers (access flags) for this class. */
@@ -114,19 +103,64 @@ public class ClassType extends ObjectType implements AttrContainer {
 
   public ClassType getSuperclass ()
   {
-    if (superClass == null && reflectClass != null
-        && this != Type.pointer_type)
-      superClass = (ClassType) make(reflectClass.getSuperclass());
+    if (superClass == null && reflectClass != null)
+      {
+        Class superReflectClass = reflectClass.getSuperclass();
+        // Neither Object nor interfaces have a superclass.
+        // I.e. equivalent to if(this!=Type.pointer_type && !isInterface()).
+        if (superReflectClass != null)
+          superClass = (ClassType) make(superReflectClass);
+      }
     return superClass;
- }
+  }
 
-  public ClassType[] getInterfaces() { return interfaces; }
+  /**
+   * @return the interfaces this class is declared to implement
+   * (not those inherited from its superclass/superinterfaces).
+   */
+  public ClassType[] getInterfaces()
+  {
+    if (interfaces == null && reflectClass != null)
+      {
+	Class[] reflectInterfaces = reflectClass.getInterfaces();
+	interfaces = new ClassType[reflectInterfaces.length];
+
+	for (int i = 0; i < reflectInterfaces.length; i++)
+	  interfaces[i] = (ClassType) Type.make(reflectInterfaces[i]);
+      }
+    return interfaces;
+  }
 
   public String getNameOrSignature() { return getName(); }
 
   public void setInterfaces (ClassType[] interfaces)
   { this.interfaces = interfaces; }
 
+  /** Add an interface to the list of implemented interfaces. */
+  public void addInterface (ClassType newInterface)
+  {
+    int oldCount;
+    if (interfaces == null || interfaces.length == 0)
+      {
+	oldCount = 0;
+	interfaces = new ClassType[1];
+      }
+    else
+      {
+	oldCount = interfaces.length;
+	for (int i = oldCount;  --i >= 0; )
+	  if (interfaces[i] == newInterface)
+	    return;
+	ClassType[] newInterfaces = new ClassType[oldCount+1];
+	System.arraycopy(interfaces, 0, newInterfaces, 0, oldCount);
+	interfaces = newInterfaces;
+      }
+    interfaces[oldCount] = newInterface;
+  }
+
+  public final boolean isInterface()
+  { return (getModifiers() & Access.INTERFACE) != 0; }
+
   public ClassType () { }
 
   public ClassType (String class_name)
@@ -153,6 +187,8 @@ public class ClassType extends ObjectType implements AttrContainer {
   /** Get the fields of this class. */
   public final Field getFields()
   {
+    if ((flags & (ADD_FIELDS_DONE|EXISTING_CLASS)) == EXISTING_CLASS)
+      addFields();
     return fields;
   }
 
@@ -161,9 +197,13 @@ public class ClassType extends ObjectType implements AttrContainer {
     return fields_count;
   }
 
-  /** Find a field with the given name,or null. */
-  public Field getField(String name)
+  /** Find a field with the given name declared in this class.
+   * @return the matching field, or null if there is no such field.
+   */
+  public Field getDeclaredField(String name)
   {
+    if ((flags & (ADD_FIELDS_DONE|EXISTING_CLASS)) == EXISTING_CLASS)
+      addFields();
     Field field;
     for (field = fields;   field != null;  field = field.next)
       {
@@ -173,6 +213,23 @@ public class ClassType extends ObjectType implements AttrContainer {
     return null;
   }
 
+  /** Find a field with the given name declared in this class or its ancestors.
+   * @return the matching field, or null if there is no such field.
+   */
+  public Field getField(String name)
+  {
+    ClassType cl = this;
+    for (;;)
+      {
+        Field field = cl.getDeclaredField(name);
+        if (field != null)
+          return field;
+        cl = cl.getSuperclass();
+        if (cl == null)
+          return null;
+      }
+  }
+
   /**
    * Add a new field to this class.
    */
@@ -194,15 +251,43 @@ public class ClassType extends ObjectType implements AttrContainer {
     field.setType(type);
     return field;
   }
+
   public final Field addField (String name, Type type, int flags)
   {
     Field field = addField (name, type);
-    field.setName(name);
-    field.type = type;
     field.flags = flags;
     return field;
   }
 
+  /** Use reflection to add all the declared fields of this class.
+   * Does not add private or package-private fields.
+   * Does not check for duplicate (already-known) fields. */
+  public void addFields()
+  {
+    Class clas = getReflectClass();
+    java.lang.reflect.Field[] fields;
+    try
+      {
+        fields = clas.getDeclaredFields();
+      }
+    catch (SecurityException ex)
+      {
+        fields = clas.getFields();
+      }
+    int count = fields.length;
+    for (int i = 0;  i < count;  i++)
+      {
+        java.lang.reflect.Field field = fields[i];
+        if (! field.getDeclaringClass().equals(clas))
+          continue;
+        int modifiers = field.getModifiers();
+        if ((modifiers & (Access.PUBLIC|Access.PROTECTED)) == 0)
+          continue;
+        addField(field.getName(), Type.make(field.getType()), modifiers);
+      }
+    flags |= ADD_FIELDS_DONE;
+  }
+
   Method methods;
   int methods_count;
   Method last_method;
@@ -248,27 +333,11 @@ public class ClassType extends ObjectType implements AttrContainer {
   public Method addMethod (String name, int flags,
 			   Type[] arg_types, Type return_type)
   {
-    Method method;
-    for (method = methods;  method != null;  method = method.next)
-      {
-	if (! name.equals(method.getName())
-	    || ! return_type.equals(method.getReturnType())
-	    || flags != method.access_flags)
-	  continue;
-	Type[] method_args = method.getParameterTypes();
-	if (arg_types == method_args)
-	  return method;
-	int i = arg_types.length;
-	if (i != method_args.length)
-	  continue;
-	while (-- i >= 0)
-	  {
-	    if (arg_types[i] != method_args[i])
-	      break;
-	  }
-	if (i < 0)
-	  return method;
-      }
+    Method method = getDeclaredMethod(name, arg_types);
+    if (method != null
+        && return_type.equals(method.getReturnType())
+        && flags == method.access_flags)
+      return method;
 
     method = new Method (this, flags);
     method.setName(name);
@@ -284,6 +353,66 @@ public class ClassType extends ObjectType implements AttrContainer {
     return meth;
   }
 
+  public Method getDeclaredMethod(String name, Type[] arg_types)
+  {
+    if ((flags & (ADD_METHODS_DONE|EXISTING_CLASS)) == EXISTING_CLASS)
+      addMethods(getReflectClass());
+    Method method;
+    for (method = methods;  method != null;  method = method.next)
+      {
+	if (! name.equals(method.getName()))
+	  continue;
+	Type[] method_args = method.getParameterTypes();
+	if (arg_types == null || arg_types == method_args)
+	  return method;
+	int i = arg_types.length;
+	if (i != method_args.length)
+	  continue;
+	while (-- i >= 0)
+	  {
+	    if (arg_types[i] != method_args[i])
+	      break;
+	  }
+	if (i < 0)
+	  return method;
+      }
+    return null;
+  }
+
+  /** Get a method with matching name and number of arguments. */
+  public Method getDeclaredMethod(String name, int argCount)
+  {
+    if ((flags & (ADD_METHODS_DONE|EXISTING_CLASS)) == EXISTING_CLASS)
+      addMethods(getReflectClass());
+    Method result = null;
+    for (Method method = methods;  method != null;  method = method.next)
+      {
+	if (name.equals(method.getName())
+	    && argCount == method.getParameterTypes().length)
+	  {
+	    if (result != null)
+	      throw new Error("ambiguoys call to getDeclaredMethod(\""
+			      + name + "\", " + argCount);
+	    result = method;
+	  }
+      }
+    return result;
+  }
+
+  public Method getMethod(String name, Type[] arg_types)
+  {
+    ClassType cl = this;
+    for (;;)
+      {
+        Method method = cl.getDeclaredMethod(name, arg_types);
+        if (method != null)
+          return method;
+        cl = cl.getSuperclass();
+        if (cl == null)
+          return null;
+      }
+  }
+
   /** Use reflection to add all the declared methods of this class.
    * Does not add constructors nor private or package-private methods.
    * Does not check for duplicate (already-known) methods.
@@ -300,7 +429,6 @@ public class ClassType extends ObjectType implements AttrContainer {
         methods = clas.getMethods();
       }
     int count = methods.length;
-    System.err.println("addMethods("+clas+") #"+ count);
     for (int i = 0;  i < count;  i++)
       {
         java.lang.reflect.Method method = methods[i];
@@ -314,14 +442,46 @@ public class ClassType extends ObjectType implements AttrContainer {
         Type[] args = new Type[j];
         while (--j >= 0)
           args[j] = Type.make(paramTypes[j]);
-        addMethod(method.getName(), modifiers,
-                  args, Type.make(method.getReturnType()));
+        Method meth = new Method (this, modifiers);
+        meth.setName(method.getName());
+        meth.arg_types = args;
+        meth.return_type = Type.make(method.getReturnType());
       }
+
+    java.lang.reflect.Constructor[] cmethods;
+    try
+      {
+        cmethods = clas.getDeclaredConstructors();
+      }
+    catch (SecurityException ex)
+      {
+        cmethods = clas.getConstructors();
+      }
+    count = cmethods.length;
+    for (int i = 0;  i < count;  i++)
+      {
+        java.lang.reflect.Constructor method = cmethods[i];
+        if (! method.getDeclaringClass().equals(clas))
+          continue;
+        int modifiers = method.getModifiers();
+        if ((modifiers & (Access.PUBLIC|Access.PROTECTED)) == 0)
+          continue;
+        Class[] paramTypes = method.getParameterTypes();
+        int j = paramTypes.length;
+        Type[] args = new Type[j];
+        while (--j >= 0)
+          args[j] = Type.make(paramTypes[j]);
+        Method meth = new Method (this, modifiers);
+        meth.setName("<init>");
+        meth.arg_types = args;
+        meth.return_type = Type.void_type;
+      }
+
+    flags |= ADD_METHODS_DONE;
   }
 
   public Method[] getMatchingMethods(String name, Type[] paramTypes, int flags)
   {
-    System.err.println(""+this+".getMatchingMethods("+name);
     int i = getMethodCount();
     int nMatches = 0;
     java.util.Vector matches = new java.util.Vector(10);
@@ -489,6 +649,39 @@ public class ClassType extends ObjectType implements AttrContainer {
     return buffer;
   }
 
+  public final boolean isSubclass(ClassType other)
+  {
+    ClassType baseClass = this;
+    while (baseClass != null)
+      {
+        if (baseClass == other)
+          return true;
+        baseClass = baseClass.getSuperclass();
+      }
+    return false;
+  }
+
+  public int compare(Type other)
+  {
+    if (other instanceof PrimType)
+      return swappedCompareResult(((PrimType) other).compare(this));
+    if (other instanceof ArrayType)
+      return swappedCompareResult(((ArrayType) other).compare(this));
+    if (! (other instanceof ClassType))
+      return -3;
+    String name = getName();
+    if (name != null && name.equals(other.getName()))
+      return 0;
+    ClassType cother = (ClassType) other;
+    if (this.isInterface() || cother.isInterface())
+      return -2;
+    if (isSubclass(cother))
+      return -1;
+    if (cother.isSubclass(this))
+      return 1;
+    return -3;
+  }
+
   public String toString()
   {
     return "ClassType " + getName();
