@@ -25,9 +25,13 @@ public class CodeAttr extends Attribute implements AttrContainer
   public final void setAttributes (Attribute attributes)
   { this.attributes = attributes; }
   LineNumbersAttr lines;
-  LocalVarsAttr locals;
+  public LocalVarsAttr locals;
 
+  // In hindsight, maintaining stack_types is more hassle than it is worth.
+  // Instead, better to just keep track of SP, which should catch most
+  // stack errors, while being more general and less hassle.  FIXME.
   Type[] stack_types;
+
   int SP;  // Current stack size (in "words")
   private int max_stack;
   private int max_locals;
@@ -105,9 +109,7 @@ public class CodeAttr extends Attribute implements AttrContainer
   public CodeAttr (Method meth)
   {
     super ("Code");
-    setContainer(meth);
-    setNext(meth.getAttributes());
-    meth.setAttributes(this);
+    addToFrontOf(meth);
     meth.code = this;
   }
 
@@ -337,7 +339,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 
   public void enterScope (Scope scope)
   {
-    scope.start_pc = PC;
+    scope.setStartPC(PC);
     locals.enterScope(scope);
   }
 
@@ -346,7 +348,7 @@ public class CodeAttr extends Attribute implements AttrContainer
     scope.start_pc = PC;
     readPC = PC;
     if (locals == null)
-      locals = new LocalVarsAttr(this);
+      locals = new LocalVarsAttr(getMethod());
     locals.enterScope(scope);
     if (locals.parameter_scope == null) 
       locals.parameter_scope= scope;
@@ -590,9 +592,9 @@ public class CodeAttr extends Attribute implements AttrContainer
      reference) to 7 (for 'short') which matches the pattern of how JVM
      opcodes typically depend on the operand type. */
 
-  private int adjustTypedOp  (Type type)
+  private int adjustTypedOp  (char sig)
   {
-    switch (type.getSignature().charAt(0))
+    switch (sig)
       {
       case 'I':  return 0;  // int
       case 'J':  return 1;  // long
@@ -606,12 +608,23 @@ public class CodeAttr extends Attribute implements AttrContainer
       }
   }
 
+  private int adjustTypedOp  (Type type)
+  {
+    return adjustTypedOp(type.getSignature().charAt(0));
+  }
+
   private void emitTypedOp (int op, Type type)
   {
     reserve(1);
     put1(op + adjustTypedOp(type));
   }
 
+  private void emitTypedOp (int op, char sig)
+  {
+    reserve(1);
+    put1(op + adjustTypedOp(sig));
+  }
+
   /** Store into an element of an array.
    * Must already have pushed the array reference, the index,
    * and the new value (in that order).
@@ -704,6 +717,7 @@ public class CodeAttr extends Attribute implements AttrContainer
     emitNewArray (element_type, 1);
   }
 
+  // We may want to deprecate this, because it depends on popType.
   private void emitBinop (int base_code)
   {
     Type type2 = popType().promote();
@@ -715,13 +729,37 @@ public class CodeAttr extends Attribute implements AttrContainer
     pushType(type1_raw);
   }
 
+  private void emitBinop (int base_code, char sig)
+  {
+    popType();
+    popType();
+    emitTypedOp(base_code, sig);
+    pushType(Type.signatureToPrimitive(sig));
+  }
+
+  private void emitBinop (int base_code, Type type)
+  {
+    popType();
+    popType();
+    emitTypedOp(base_code, type);
+    pushType(type);
+  }
+
   // public final void emitIntAdd () { put1(96); popType();}
   // public final void emitLongAdd () { put1(97); popType();}
   // public final void emitFloatAdd () { put1(98); popType();}
   // public final void emitDoubleAdd () { put1(99); popType();}
 
+  public final void emitAdd(char sig) { emitBinop (96, sig); }
+  public final void emitAdd(PrimType type) { emitBinop (96, type); }
+  /** @deprecated */
   public final void emitAdd () { emitBinop (96); }
+
+  public final void emitSub(char sig) { emitBinop (100, sig); }
+  public final void emitSub(PrimType type) { emitBinop (100, type); }
+  /** @deprecated */
   public final void emitSub () { emitBinop (100); }
+
   public final void emitMul () { emitBinop (104); }
   public final void emitDiv () { emitBinop (108); }
   public final void emitRem () { emitBinop (112); }
@@ -881,6 +919,18 @@ public class CodeAttr extends Attribute implements AttrContainer
     emitFieldop(field, 181);  // putfield
   }
 
+  /** Comptes the number of stack words taken by a list of types. */
+  private int words(Type[] types)
+  {
+    int res = 0;
+    for (int i=types.length; --i >= 0; )
+      if (types[i].size > 4)
+       res+=2;
+      else
+       res++;
+    return res;
+  }
+
   public void emitInvokeMethod (Method method, int opcode)
   {
     reserve(opcode == 185 ? 5 : 3);
@@ -895,7 +945,7 @@ public class CodeAttr extends Attribute implements AttrContainer
     putIndex2(getConstants().addMethodRef(method));
     if (opcode == 185)  // invokeinterface
       {
-	put1(arg_count);
+	put1(words(method.arg_types)+1); // 1 word for 'this'
 	put1(0);
       }
     while (--arg_count >= 0)
@@ -904,6 +954,18 @@ public class CodeAttr extends Attribute implements AttrContainer
       pushType(method.return_type);
   }
 
+  public void emitInvoke (Method method)
+  {
+    int opcode;
+    if ((method.access_flags & Access.STATIC) != 0)
+      opcode = 184;   // invokestatic
+    else if (getMethod().classfile.isInterface())
+      opcode = 185;   // invokeinterface
+    else
+      opcode = 182;   // invokevirtual
+    emitInvokeMethod(method, opcode);
+  }
+
   /** Compile a virtual method call.
    * The stack contains the 'this' object, followed by the arguments in order.
    * @param method the method to invoke virtually
@@ -972,7 +1034,6 @@ public class CodeAttr extends Attribute implements AttrContainer
     setUnreachable();
   }
 
-  //public final void compile_goto_ifeq (Label label, boolean invert)
   public final void emitGotoIfEq (Label label, boolean invert)
   {
     Type type2 = popType().promote();
@@ -1002,7 +1063,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 	     && (sig2 == 'L' || sig2 == '['))
       opcode = 165;  // if_acmpeq (inverted: if_acmpne)
     else
-      throw new Error ("non-matching types to compile_goto_ifeq");
+      throw new Error ("non-matching types to emitGotoIfEq");
     if (invert)
       opcode++;
     emitTransfer (label, opcode);
@@ -1096,6 +1157,12 @@ public class CodeAttr extends Attribute implements AttrContainer
     emitIfCompare1(153); // ifeq
   }
 
+  /** Compile start of conditional:  if (x <= 0) */
+  public final void emitIfIntLEqZero()
+  {
+    emitIfCompare1(157); // ifgt
+  }
+
   /** Compile start of a conditional:  if (!(x OPCODE null)) ...
    * The value of x must already have been pushed and must be of
    * reference type. */
@@ -1215,7 +1282,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 
   public final void emitIfThen ()
   {
-    new IfState(this);
+    new IfState(this, null);
   }
 
   /** Compile start of else clause. */
@@ -1226,15 +1293,23 @@ public class CodeAttr extends Attribute implements AttrContainer
     if_stack.end_label = end_label;
     if (reachableHere ())
       {
-	int stack_growth = SP-if_stack.start_stack_size;
-	if_stack.then_stacked_types = new Type[stack_growth];
-	System.arraycopy (stack_types, if_stack.start_stack_size,
-			  if_stack.then_stacked_types, 0, stack_growth);
+	int growth = SP-if_stack.start_stack_size;
+	if_stack.stack_growth = growth;
+	if (growth >= 0)
+	  {
+	    if_stack.then_stacked_types = new Type[growth];
+	    System.arraycopy (stack_types, if_stack.start_stack_size,
+			      if_stack.then_stacked_types, 0, growth);
+	  }
+	else
+	  if_stack.then_stacked_types = new Type[0];  // ???
 	emitGoto (end_label);
       }
-    while (SP != if_stack.start_stack_size)
+    while (SP > if_stack.start_stack_size)
       popType();
-    else_label.define (this);
+    SP = if_stack.start_stack_size;
+    if (else_label != null)
+      else_label.define (this);
     if_stack.doing_else = true;    
   }
 
@@ -1251,12 +1326,13 @@ public class CodeAttr extends Attribute implements AttrContainer
     else if (if_stack.then_stacked_types != null)
       {
 	int then_clause_stack_size
-	  = if_stack.start_stack_size + if_stack.then_stacked_types.length;
+	  = if_stack.start_stack_size + if_stack.stack_growth;
 	if (! reachableHere ())
 	  {
-	    System.arraycopy (if_stack.then_stacked_types, 0,
-			      stack_types, if_stack.start_stack_size,
-			      if_stack.then_stacked_types.length);
+	    if (if_stack.stack_growth > 0)
+	      System.arraycopy (if_stack.then_stacked_types, 0,
+				stack_types, if_stack.start_stack_size,
+				if_stack.stack_growth);
 	    SP = then_clause_stack_size;
 	  }
 	else if (SP != then_clause_stack_size)
@@ -1267,7 +1343,8 @@ public class CodeAttr extends Attribute implements AttrContainer
     else if (unreachable_here)
       make_unreachable = true;
 
-    if_stack.end_label.define (this);
+    if (if_stack.end_label != null)
+      if_stack.end_label.define (this);
     if (make_unreachable)
       setUnreachable();
     // Pop the if_stack.
@@ -1283,6 +1360,8 @@ public class CodeAttr extends Attribute implements AttrContainer
       {
 	char to_sig0 = to_sig.charAt(0);
 	char from_sig0 = from_sig.charAt(0);
+	if (from_sig0 == to_sig0)
+	  return;
 	if (from.size < 4)
 	  from_sig0 = 'I';
 	if (to.size < 4)
@@ -1332,7 +1411,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 	  }
       }
     if (op < 0)
-      throw new Error ("unsupported Method.compile_convert");
+      throw new Error ("unsupported CodeAttr.emitConvert");
     reserve(1);
     popType();
     put1(op);
@@ -1656,6 +1735,8 @@ public class CodeAttr extends Attribute implements AttrContainer
   public void assignConstants (ClassType cl)
   {
     super.assignConstants(cl);
+    if (locals != null && locals.container == null && ! locals.isEmpty())
+      locals.addToFrontOf(this);
     Attribute.assignConstants(this, cl);
   }
 
