@@ -74,7 +74,7 @@ public class Scope
   {
     var.start_pc = code == null ? 0 : code.PC;
     addVariable (var);
-    if (var.isSimple ())
+    if (var.isSimple() && code != null)
       var.allocateLocal(code);
   }
 
@@ -100,6 +100,13 @@ public class Scope
     return true;
   }
 
+  public void setStartPC(int PC)
+  {
+    start_pc = PC;
+    for (Variable var = firstVar ();  var != null;  var = var.nextVar ())
+      var.start_pc = PC;
+  }
+
   /**
    * Search by name for a Variable in this Scope (only).
    * @param name name to search for
