@@ -9,10 +9,8 @@ import java.io.*;
 
 public class Compilation
 {
-  /** Set of PrimProcedures generated for visible top-level procedures. */
+  /** Set of visible top-level LambdaExps that need apply methods. */
   Vector applyMethods = new Vector();
-  /** Number of the different kinds of applyMethods. */
-  int[] applyMethodsCount = new int[6]; // apply0..apply4+applyN
   /** Used by LambdaExp.getSelectorValue if need to allocate new selector. */
   int maxSelectorValue;
 
@@ -20,7 +18,7 @@ public class Compilation
   public ClassType mainClass;
 
   public LambdaExp curLambda;
-  public Declaration thisDecl;
+  public Variable thisDecl;
 
   /** If true, minimize the number of classes generated.
    * Do this even if it makes things a little slower. */
@@ -47,7 +45,6 @@ public class Compilation
   SwitchState fswitch;
 
   Field fswitchIndex;
-  Variable argsArray;
   /** The actual parameter of the current CallFrame.step(CallStack) method. */
   Variable callStackContext;
 
@@ -58,45 +55,56 @@ public class Compilation
   static public ClassType javaStringType = typeString;
   static public ClassType scmSymbolType = typeString;
   static public ClassType scmKeywordType = ClassType.make("gnu.expr.Keyword");
-  static public ClassType scmSequenceType = ClassType.make("kawa.lang.Sequence");
+  static public ClassType scmSequenceType = ClassType.make("gnu.kawa.util.Sequence");
   static public ClassType javaIntegerType = ClassType.make("java.lang.Integer");
-  static public ClassType scmListType = ClassType.make("kawa.lang.List");
-  static public ClassType typePair = ClassType.make("kawa.lang.Pair");
+  static public ClassType scmListType = ClassType.make("gnu.kawa.util.LList");
+  static public ClassType typePair = ClassType.make("gnu.kawa.util.Pair");
   static public ClassType scmPairType = typePair;
   static public ClassType scmUndefinedType = ClassType.make("gnu.expr.Undefined");
-  public static final ArrayType objArrayType = new ArrayType (typeObject);
-  public static final ArrayType symbolArrayType = new ArrayType(scmSymbolType);
+  public static final ArrayType objArrayType = ArrayType.make(typeObject);
+  public static final ArrayType symbolArrayType= ArrayType.make(scmSymbolType);
   static public ClassType scmNamedType = ClassType.make("gnu.mapping.Named");
+  static public ClassType typeRunnable = ClassType.make("java.lang.Runnable");
   static public ClassType typeProcedure
     = ClassType.make("gnu.mapping.Procedure");
-  static public ClassType scmInterpreterType
-    = ClassType.make("kawa.lang.Interpreter");
+  static public ClassType typeInterpreter
+    = ClassType.make("gnu.expr.Interpreter");
+  static public ClassType typeMacro
+    = ClassType.make("kawa.lang.Macro");
   static public ClassType typeEnvironment
     = ClassType.make("gnu.mapping.Environment");
-  static public ClassType typeBinding = ClassType.make("gnu.mapping.Binding");
-  static public final Method getBindingMethod
-    = typeBinding.addMethod("get", Type.typeArray0,
+  static public ClassType typeLocation
+    = ClassType.make("gnu.mapping.Location");
+  static public ClassType typeBinding
+    = ClassType.make("gnu.mapping.Binding", typeLocation);
+  static public ClassType typeBinding2
+    = ClassType.make("gnu.mapping.Binding2", typeBinding);
+  static final Field functionValueBinding2Field
+    = typeBinding2.addField ("functionValue", Type.pointer_type,Access.PUBLIC);
+  static public final Method getLocationMethod
+    = typeLocation.addMethod("get", Type.typeArray0,
 			    Type.pointer_type, Access.PUBLIC);
   static public final Method getProcedureBindingMethod
     = typeBinding.addMethod("getProcedure", Type.typeArray0,
 			    typeProcedure, Access.PUBLIC);
   static public final Field trueConstant
-    = scmInterpreterType.addField ("trueObject", scmBooleanType,
-				    Access.PUBLIC|Access.STATIC); 
+    = scmBooleanType.addField ("TRUE", scmBooleanType,
+			       Access.PUBLIC|Access.STATIC); 
   static public final Field falseConstant
-    = scmInterpreterType.addField ("falseObject", scmBooleanType,
-				    Access.PUBLIC|Access.STATIC);
+    = scmBooleanType.addField ("FALSE", scmBooleanType,
+			       Access.PUBLIC|Access.STATIC);
   static final Field voidConstant
-  = scmInterpreterType.addField ("voidObject", typeObject,
-				  Access.PUBLIC|Access.STATIC);
+    = typeInterpreter.addField ("voidObject", typeObject,
+				Access.PUBLIC|Access.STATIC);
   static final Field undefinedConstant
-  = scmInterpreterType.addField ("undefinedObject", scmUndefinedType,
-				    Access.PUBLIC|Access.STATIC);
+    = typeInterpreter.addField ("undefinedObject", scmUndefinedType,
+				Access.PUBLIC|Access.STATIC);
   static final Field nameField
-  = scmNamedType.addField ("sym_name", scmSymbolType, Access.PUBLIC);
+    = typeProcedure.addField ("sym_name", scmSymbolType, Access.PROTECTED);
   static Method initIntegerMethod;
   static Method lookupGlobalMethod;
   static Method defineGlobalMethod;
+  static Method defineFunctionMethod;
   static Method putGlobalMethod;
   static Method makeListMethod;
   
@@ -107,6 +115,8 @@ public class Compilation
   static public final Method getBindingEnvironmentMethod
     = typeEnvironment.addMethod("getBinding", string1Arg,
 				typeBinding, Access.PUBLIC);
+  public static final Method getBinding2Method
+    = typeBinding2.getDeclaredMethod("getBinding2", 2);
 
   static {
     Type[] makeListArgs = { objArrayType, Type.int_type };
@@ -125,6 +135,9 @@ public class Compilation
     defineGlobalMethod
       = typeEnvironment.addMethod ("define_global", symObjArgs,
 				   Type.void_type,Access.PUBLIC|Access.STATIC);
+    defineFunctionMethod
+      = typeEnvironment.addMethod ("defineFunction", symObjArgs,
+				   Type.void_type,Access.PUBLIC|Access.STATIC);
     putGlobalMethod
       = typeEnvironment.addMethod ("put_global", symObjArgs,
 				   Type.void_type,Access.PUBLIC|Access.STATIC);
@@ -197,46 +210,14 @@ public class Compilation
     = ClassType.make("gnu.mapping.ProcedureN", typeProcedure);
   public static ClassType typeModuleBody
     = ClassType.make("gnu.expr.ModuleBody", typeProcedure0);
+  public static ClassType typeApplet = ClassType.make("java.applet.Applet");
 
   public static ClassType typeModuleMethod
   = ClassType.make("gnu.expr.ModuleMethod", typeProcedureN);
-  private static Type[] apply0argsModule = { typeModuleMethod };
-  private static Type[] apply1argsModule = { typeModuleMethod, typeObject };
-  private static Type[] apply2argsModule = { typeModuleMethod, typeObject,
-                                             typeObject };
-  private static Type[] apply3argsModule = { typeModuleMethod, typeObject,
-                                             typeObject, typeObject };
-  private static Type[] apply4argsModule
-  = { typeModuleMethod, typeObject, typeObject, typeObject,typeObject };
-  private static Type[] applyNargsModule = { typeModuleMethod, objArrayType };
-  static Type[][] applyArgsModule = { apply0argsModule, apply1argsModule,
-                                      apply2argsModule, apply3argsModule,
-                                      apply4argsModule, applyNargsModule } ;
-  static Method apply0ModuleMethod
-  = typeModuleBody.addMethod ("apply0", apply0argsModule,
-                              typeObject, Access.PUBLIC);
-  static Method apply1ModuleMethod
-  = typeModuleBody.addMethod ("apply1", apply1argsModule,
-                              typeObject, Access.PUBLIC);
-  static Method apply2ModuleMethod
-  = typeModuleBody.addMethod ("apply2", apply2argsModule,
-                              typeObject, Access.PUBLIC);
-  static Method apply3ModuleMethod
-  = typeModuleBody.addMethod ("apply3", apply3argsModule,
-                              typeObject, Access.PUBLIC);
-  static Method apply4ModuleMethod
-  = typeModuleBody.addMethod ("apply4", apply4argsModule,
-                              typeObject, Access.PUBLIC);
-  static Method applyNModuleMethod
-  = typeModuleBody.addMethod ("applyN", applyNargsModule,
-                              typeObject, Access.PUBLIC);
-  public static Method[] applyModuleMethods = {
-    apply0ModuleMethod, apply1ModuleMethod, apply2ModuleMethod,
-    apply3ModuleMethod, apply4ModuleMethod, applyNModuleMethod };
-
-  static public final Field selectorOfModuleMethod
-    = typeModuleMethod.addField ("selector", Type.int_type,
-                                 Access.PUBLIC|Access.FINAL); 
+  public static ClassType typeApplyMethodProc
+  = ClassType.make("gnu.mapping.ApplyMethodProc", typeProcedureN);
+  public static ClassType typeApplyMethodContainer
+  = ClassType.make("gnu.mapping.ApplyMethodContainer");
 
   /* Classes, fields, and methods used wgen usingCPStyle". */
   public static ClassType typeCallStack
@@ -244,8 +225,10 @@ public class Compilation
   public static Method popCallStackMethod
     = typeCallStack.addMethod("pop", apply0args, Type.void_type,
 			      Access.PUBLIC);
-  public static Field noArgsProcedureField
-    = typeProcedure.addField("noArgs", objArrayType,
+  public static ClassType typeValues
+    = ClassType.make("gnu.mapping.Values");
+  public static Field noArgsField
+    = typeValues.addField("noArgs", objArrayType,
 				Access.PUBLIC|Access.STATIC);
   public static Field valueCallStackField
     = typeCallStack.addField("value", Type.pointer_type, Access.PUBLIC);
@@ -276,14 +259,42 @@ public class Compilation
 
   Hashtable literalTable;
   int literalsCount;
-  Literal literalsChain;
+
+  /** Rembembers stuff to do in <init> of main class. */
+  Initializer initChain;
+
+  /** Rembembers stuff to do in <clinit> of main class. */
+  Initializer clinitChain;
 
   public static boolean generateMainDefault = false;
   /** True if we should generate a main(String[]) method. */
   public boolean generateMain = generateMainDefault;
 
+  public static boolean generateAppletDefault = false;
+  /** True if we should generate an Applet. */
+  public boolean generateApplet = generateAppletDefault;
+
+  public static final ClassType getMethodProcType(ClassType modClass)
+  {
+    return modClass.getSuperclass().isSubtype(typeProcedure) ? typeModuleMethod
+      : typeApplyMethodProc;
+  }
+
+  public final ClassType getModuleSuperType(ModuleExp module)
+  {
+    ClassType sup = module.getSuperType();
+    return sup != null ? sup : generateApplet ? typeApplet : typeModuleBody;
+  }
+
+  public Interpreter getInterpreter()
+  {
+    return Interpreter.defaultInterpreter;  // For now.  FIXME.
+  }
+
   public Literal findLiteral (Object value)
   {
+    if (value == null)
+      return Literal.nullLiteral;
     Literal literal = (Literal) literalTable.get (value);
     if (literal != null)
       {
@@ -316,16 +327,21 @@ public class Compilation
 	  {
 	    Object[] array = (Object[]) value;
 	    int len = array.length;
-	    literal = new Literal (value, objArrayType, this);
-	    for (int i = array.length;  --i >= 0; )
+	    if (len == 0 && ! (value instanceof String[]))
+	      literal = new Literal(value, noArgsField, this);
+	    else
 	      {
-		Object element = array[i];
-		if (element != null)
-		  findLiteral(element);
+		literal = new Literal (value, objArrayType, this);
+		for (int i = len;  --i >= 0; )
+		  {
+		    Object element = array[i];
+		    if (element != null)
+		      findLiteral(element);
+		  }
 	      }
 	  }
 	else
-	  literal = new Literal (value, typeObject, this);
+	  literal = new Literal (value, Type.make(value.getClass()), this);
       }
     return literal;
   }
@@ -336,10 +352,16 @@ public class Compilation
    */
   public void emitLiteral (Object value)
   {
-    Literal literal = (Literal) literalTable.get (value);
-    if (literal == null)
-      throw new Error ("emitLiteral called without previous findLiteral");
-    literal.emit (this, false);
+    gnu.bytecode.CodeAttr code = getCode();
+    if (value == null)
+      code.emitPushNull();
+    else
+      {
+	Literal literal = (Literal) literalTable.get (value);
+	if (literal == null)
+	  throw new Error ("emitLiteral called without previous findLiteral");
+	literal.emit (this, false);
+      }
   }
 
   /** Emit code to "evaluate" a compile-time constant.
@@ -369,7 +391,7 @@ public class Compilation
     if (target instanceof ConditionalTarget)
       {
 	ConditionalTarget ctarg = (ConditionalTarget) target;
-	getCode().emitGoto(IfExp.is_true(value) ? ctarg.ifTrue
+	getCode().emitGoto(getInterpreter().isTrue(value) ? ctarg.ifTrue
 			   : ctarg.ifFalse);
 	return;
       }
@@ -420,19 +442,27 @@ public class Compilation
 		// should print an ERROR.
 	      }
 	  }
+        try
+          {
+            value = type.coerceFromObject(value);
+          }
+        catch (Exception ex)
+          {
+            error('w', "cannot convert literal (of type "
+                  + value.getClass().getName() + ") to "
+                  + type.getName());
+          }
       }
     compileConstant(value);
-    target.compileFromStack(this, Type.pointer_type);
+    target.compileFromStack(this,
+                            value == null ? target.getType()
+                            : Type.make(value.getClass()));
   }
 
-  private void dumpLiterals ()
+  private void dumpInitializers (Initializer inits)
   {
-    for (Literal literal = literalsChain;  literal != null;
-	 literal = literal.next)
-      {
-	if ((literal.flags & Literal.INITIALIZED) == 0)
-	  literal.emit (this, true);
-      }
+    for (Initializer init = inits;  init != null;  init = init.next)
+      init.emit(this);
   }
 
   /** Search this Compilation for a ClassType with a given name.
@@ -464,7 +494,7 @@ public class Compilation
 	// This function is probably not quite enough ...
 	// (Note the verifier may be picky about class names.)
 	if (Character.isLowerCase (ch) || Character.isUpperCase (ch)
-	    || Character.isDigit (ch) || ch == '_' || ch == '$')
+	    || (Character.isDigit (ch) && i > 0) || ch == '_' || ch == '$')
 	  mangled.append(ch);
 	else if (ch == '!')
 	  mangled.append("_B");
@@ -490,7 +520,7 @@ public class Compilation
 		next = i + 1 < len ? name.charAt(i+1) : '\0';
 		if (Character.isLowerCase(next))
 		  {
-		    mangled.append("To");
+		    mangled.append(i == 1 ? "to" : "To");
 		    next = Character.toTitleCase(next);
 		    mangled.append(next);
 		    i++;
@@ -509,14 +539,47 @@ public class Compilation
 	  }
 	else
 	  {
-	    mangled.append ('_');
-	    mangled.append (Character.forDigit ((ch >> 12) & 15, 16));
-	    mangled.append (Character.forDigit ((ch >>  8) & 15, 16));
-	    mangled.append (Character.forDigit ((ch >>  4) & 15, 16));
-	    mangled.append (Character.forDigit ((ch      ) & 15, 16));
+	    switch (ch)
+	      {
+	      case '+':  mangled.append("$Pl");  break;
+	      case '-':  mangled.append("$Mn");  break;
+	      case '*':  mangled.append("$St");  break;
+	      case '/':  mangled.append("$Sl");  break;
+	      case '=':  mangled.append("$Eq");  break;
+	      case '<':  mangled.append("$Ls");  break;
+	      case '>':  mangled.append("$Gr");  break;
+	      case '@':  mangled.append("$At");  break;
+	      case '~':  mangled.append("$Tl");  break;
+	      case '%':  mangled.append("$Pc");  break;
+	      case '.':  mangled.append("$Dt");  break;
+	      case ',':  mangled.append("$Cm");  break;
+	      case '(':  mangled.append("$LP");  break;
+	      case ')':  mangled.append("$RP");  break;
+	      case '[':  mangled.append("$LB");  break;
+	      case ']':  mangled.append("$RB");  break;
+	      case '{':  mangled.append("$LC");  break;
+	      case '}':  mangled.append("$RC");  break;
+	      case '\'': mangled.append("$Sq");  break;
+	      case '"':  mangled.append("$Dq");  break;
+	      case '&':  mangled.append("$Am");  break;
+	      case '#':  mangled.append("$Nm");  break;
+	      case '?':  mangled.append("$Qu");  break;
+	      case '!':  mangled.append("$Ex");  break;
+	      case ':':  mangled.append("$Cl");  break;
+	      case ';':  mangled.append("$SC");  break;
+	      case '^':  mangled.append("$Up");  break;
+	      case '|':  mangled.append("$VB");  break;
+	      default:
+		mangled.append('$');
+		mangled.append(Character.forDigit ((ch >> 12) & 15, 16));
+		mangled.append(Character.forDigit ((ch >>  8) & 15, 16));
+		mangled.append(Character.forDigit ((ch >>  4) & 15, 16));
+		mangled.append(Character.forDigit ((ch      ) & 15, 16));
+	      }
 	  }
       }
-    return mangled.toString ();
+    String mname = mangled.toString ();
+    return mname.equals(name) ? name : mname;
   }
 
   /** Generate an unused class name.
@@ -595,13 +658,21 @@ public class Compilation
     else
       {
 	type = new ClassType(name);
-	ClassType superType
-	  = lexp.isModuleBody () ? typeModuleBody
-	  : usingCPStyle ? typeCallFrame
-	  : lexp.isHandlingTailCalls() ? typeCpsProcedure
-	  : (lexp.min_args != lexp.max_args || lexp.min_args > 4)
-	  ? typeProcedureN
-	  : typeProcedureArray[lexp.min_args];
+	ClassType superType;
+	if (lexp.isModuleBody ())
+	  {
+	    ModuleExp module = (ModuleExp) lexp;
+	    superType = getModuleSuperType(module);
+	    ClassType[] interfaces = module.getInterfaces();
+	    if (interfaces != null)
+	      type.setInterfaces(interfaces);
+	  }
+	else
+	  superType = (usingCPStyle ? typeCallFrame
+		       : lexp.isHandlingTailCalls() ? typeCpsProcedure
+		       : (lexp.min_args != lexp.max_args || lexp.min_args > 4)
+		       ? typeProcedureN
+		       : typeProcedureArray[lexp.min_args]);
 	type.setSuper (superType);
       }
 
@@ -628,6 +699,16 @@ public class Compilation
     code.emitPushThis();
     code.emitInvokeSpecial(superConstructor);
 
+    if (curClass == mainClass)
+      {
+	Initializer init;
+	while ((init = initChain) != null)
+	  {
+	    initChain = init.next;
+	    init.emit(this);
+	  }
+      }
+
     // If immediate, we cannot set the function name in the constructor,
     // since setLiterals has not been called yet (ecept for nested functions).
     if (lexp != null && lexp.name != null && !immediate)
@@ -636,12 +717,276 @@ public class Compilation
 	compileConstant (lexp.name);
 	code.emitPutField(nameField);
       }
+
+    if (curClass == mainClass) // lexp.isModuleBody())
+      {
+	code = getCode();
+	int numGlobals = bindingFields.size();
+	if (numGlobals > 0)
+	  {
+	    code.emitInvokeStatic(getCurrentEnvironmentMethod);
+	    java.util.Enumeration e = bindingFields.keys();
+	    while (e.hasMoreElements())
+	      {
+		String id = (String) e.nextElement();
+		Field fld = (Field) bindingFields.get(id);
+		if (--numGlobals > 0)
+		  code.emitDup(1);
+		code.emitPushString(id);
+		if (getInterpreter().hasSeparateFunctionNamespace())
+		  code.emitInvokeStatic(getBinding2Method);
+		else
+		  code.emitInvokeVirtual(getBindingEnvironmentMethod);
+		code.emitPutStatic(fld);
+	      }
+	  }
+      }
+
     code.emitReturn();
     method = save_method;
     curClass = save_class;
     return constructor_method;
   }
 
+  /** Generate ModuleBody's apply0 .. applyN methods.
+   * Use the applyMethods vector, which contains methods that implement
+   * the (public, readable) methods of the current module. */
+  public void generateApplyMethods()
+  {
+    int numApplyMethods = applyMethods.size();
+    if (numApplyMethods == 0)
+      return;
+    boolean generateApplyMethodContainer
+      = ! (curClass.getSuperclass().isSubtype(typeProcedure));
+    ClassType procType
+      = generateApplyMethodContainer ? typeApplyMethodProc : typeModuleMethod;
+    if (generateApplyMethodContainer)
+      curClass.addInterface(typeApplyMethodContainer);
+    Method save_method = method;
+    CodeAttr code = null;
+    for (int i = 0;  i < 6; i++)
+      {
+	// If i < 5, generate the method named ("apply"+i);
+	// else generate "applyN".
+	boolean needThisApply = false;
+	SwitchState aswitch = null;
+	String mname = null;
+	Type[] applyArgs = null;
+
+	for (int j = numApplyMethods;  --j >= 0; )
+	  {
+	    LambdaExp source = (LambdaExp) applyMethods.elementAt(j);
+	    // Select the subset of source.primMethods[*] that are suitable
+	    // for the current apply method.
+	    Method[] primMethods = source.primMethods;
+	    int numMethods = primMethods.length;
+	    boolean varArgs = source.max_args < 0
+	      || source.max_args >= source.min_args + numMethods;
+	    int methodIndex;
+	    boolean skipThisProc = false;
+	    if (i < 5) // Handling apply0 .. apply4
+	      {
+		methodIndex = i - source.min_args;
+		if (methodIndex < 0 || methodIndex >= numMethods
+		    || (methodIndex == numMethods - 1 && varArgs))
+		  skipThisProc = true;
+		numMethods = 1;
+		varArgs = false;
+	      }
+	    else // Handling applyN
+	      {
+		methodIndex = 5 - source.min_args;
+		if (methodIndex > 0 && numMethods <= methodIndex && ! varArgs)
+		  skipThisProc = true;
+		methodIndex = numMethods-1;
+	      }
+	    if (skipThisProc && ! generateApplyMethodContainer)
+	      continue;
+	    if (! needThisApply)
+	      {
+		// First LambdaExp we seen suitable for this i.
+		if (i < 5)
+		  {
+		    mname =  "apply"+i;
+		    applyArgs = new Type[i + 1];
+		    for (int k = i;  k > 0;  k--)
+		      applyArgs[k] = typeObject;
+		  }
+		else
+		  {
+		    mname = "applyN";
+		    applyArgs = new Type[2];
+		    applyArgs[1] = objArrayType;
+		  }
+		applyArgs[0] = procType;
+		method = curClass.addMethod (mname, applyArgs,
+					     Type.pointer_type,
+					     Access.PUBLIC);
+		method.init_param_slots();
+		code = getCode();
+
+		code.emitLoad(code.getArg(1)); // method
+		code.emitGetField(procType.getDeclaredField("selector"));
+		aswitch = new SwitchState(code);
+
+		needThisApply = true;
+	      }
+	    if (skipThisProc && generateApplyMethodContainer)
+	      continue;
+
+	    aswitch.addCase(source.getSelectorValue(this), code);
+
+	    Method primMethod = primMethods[methodIndex];
+	    Type[] primArgTypes = primMethod.getParameterTypes();
+	    int nargs = primArgTypes.length;
+	    int singleArgs = varArgs ? (nargs - 1) : nargs;
+	    Variable counter = null;
+	    int pendingIfEnds = 0;
+
+	    if (i > 4 && numMethods > 1)
+	      {
+		counter = code.addLocal(Type.int_type);
+		code.emitLoad(code.getArg(2));
+		code.emitArrayLength();
+		if (source.min_args != 0)
+		  {
+		    code.emitPushInt(source.min_args);
+		    code.emitSub(Type.int_type);
+		  }
+		code.emitStore(counter);
+	      }
+
+	    int needsThis = primMethod.getStaticFlag() ? 0 : 1;
+	    if (needsThis > 0)
+	      code.emitPushThis();
+
+	    for (int k = 0; k < singleArgs;  k++)
+	      {
+		if (counter != null && k >= source.min_args)
+		  {
+		    code.emitLoad(counter);
+		    code.emitIfIntLEqZero();
+		    code.emitInvoke(primMethods[k - source.min_args]);
+		    code.emitElse();
+		    pendingIfEnds++;
+		    code.emitInc(counter, (short) (-1));
+		  }
+
+		if (i > 4) // applyN method
+		  {
+		    // Load Object[]args value:
+		    code.emitLoad(code.getArg(2));
+		    code.emitPushInt(k);
+		    code.emitArrayLoad(Type.pointer_type);
+		  }
+		else // apply'i method
+		  code.emitLoad(code.getArg(k + 2));
+		Type ptype = primArgTypes[k];
+		if (ptype != Type.pointer_type)
+		  CheckedTarget.emitCheckedCoerce(this, source,
+						  k, ptype);
+	      }
+
+	    if (varArgs)
+	      {
+		Type lastArgType = primArgTypes[singleArgs];
+		if (lastArgType instanceof ArrayType)
+		  {
+		    Type elType
+		      = ((ArrayType) lastArgType).getComponentType();
+		    boolean mustConvert
+		      = ! "java.lang.Object".equals(elType.getName());
+		    if (singleArgs == 0 && ! mustConvert)
+		      code.emitLoad(code.getArg(2)); // load args array.
+		    else
+		      {
+			code.pushScope();
+			if (counter == null)
+			  {
+			    counter = code.addLocal(Type.int_type);
+			    code.emitLoad(code.getArg(2));
+			    code.emitArrayLength();
+			    if (singleArgs != 0)
+			      {
+				code.emitPushInt(singleArgs);
+				code.emitSub(Type.int_type);
+			      }
+			    code.emitStore(counter);
+			  }
+			code.emitLoad(counter);
+			code.emitNewArray(elType);
+			Label testLabel = new Label(code);
+			code.emitGoto(testLabel);
+			Label loopTopLabel = new Label(code);
+			loopTopLabel.define(code);
+
+			code.emitDup(1); // new array
+			code.emitLoad(counter);
+			code.emitLoad(code.getArg(2));
+			code.emitLoad(counter);
+			if (singleArgs != 0)
+			  {
+			    code.emitPushInt(singleArgs);
+			    code.emitAdd(Type.int_type);
+			  }
+			code.emitArrayLoad(Type.pointer_type);
+			if (mustConvert)
+			  {
+			    CheckedTarget.emitCheckedCoerce
+			      (this, source, source.getName(), -1, elType);
+			  }
+			code.emitArrayStore(elType);
+			testLabel.define(code);
+			code.emitInc(counter, (short) (-1));
+			code.emitLoad(counter);
+			code.emitGotoIfIntGeZero(loopTopLabel);
+			code.popScope();	
+		      }
+		  }
+		else if ("gnu.kawa.util.LList".equals
+			 (lastArgType.getName()))
+		  {	
+		    code.emitLoad(code.getArg(2)); // load args array.
+		    code.emitPushInt(singleArgs);
+		    code.emitInvokeStatic(Compilation.makeListMethod);
+		  }
+		else
+		  throw new RuntimeException("unsupported #!rest type");
+              }
+
+	    code.emitInvoke(primMethod);
+	    while (--pendingIfEnds >= 0)
+	      code.emitFi();
+	    Target.pushObject.compileFromStack(this,
+					       primMethod.getReturnType());
+	    code.emitReturn();
+          }
+	if (needThisApply)
+	  {
+	    aswitch.addDefault(code);
+	    int nargs = i > 4 ? 2 : i + 1;
+	      nargs++;
+	    for (int k = generateApplyMethodContainer ? 1 : 0;
+		 k < nargs;  k++)
+	      code.emitLoad(code.getArg(k));
+	    if (generateApplyMethodContainer)
+	      {
+		mname = mname + "Default";
+		Method defMethod
+		  = typeApplyMethodProc.getDeclaredMethod(mname, applyArgs);
+		code.emitInvokeStatic(defMethod);
+	      }
+	    else
+	      {
+		code.emitInvokeSpecial(curClass.getSuperclass().getDeclaredMethod(mname, applyArgs));
+	      }
+	    code.emitReturn();
+	    aswitch.finish(code);
+	  }
+      }
+    method = save_method;
+  }
+
   /** Compiles a function to a class. */
   public final ClassType addClass (LambdaExp lexp)
   {
@@ -699,59 +1044,21 @@ public class Compilation
       }
 
     Expression body = lexp.body;
-    Declaration heapFrame = lexp.heapFrame;
-
-    // Is this worth keeping around any more?   FIXME
-    if (lexp.min_args == lexp.max_args && ! lexp.isModuleBody ()
-	&& ! lexp.getImportsLexVars() && ! lexp.getNeedsStaticLink()
-	&& ! lexp.isHandlingTailCalls())
-      {
-	Expression[] args = new Expression[lexp.max_args];
-
-	String methodJavaName = lexp.getJavaName();
-	if (methodJavaName.equals(curClass.getName()))
-	  methodJavaName = "apply";
-	Method method = lexp.addMethodFor(curClass, methodJavaName, null);
-	this.method = method;
-	method.initCode();
-	code = getCode();
-	lexp.allocParameters(this, null);
-	lexp.enterFunction(this, null);
-	Type rtype = method.getReturnType();
-	body.compileWithPosition(this, Target.returnValue(rtype));
-	lexp.compileEnd(this);
-
-	// Set up for compiling regular virtual applyX method.
-	// It just calls the static method we just finished with.
-	Variable var = lexp.firstVar ();
-	lexp.scope = new Scope();
-	int itype = 0;
-	for ( ;  var != null; var = var.nextVar ())
-	  {
-	    if (! var.isParameter() || var.isArtificial())
-	      continue;
-	    String vname = var.getName();
-	    Declaration decl = lexp.addDeclaration(vname);
-	    decl.setParameter(true);
-	    if (var.isArtificial())
-	      decl.setArtificial(true);
-	    else
-	      args[itype++] = new ReferenceExp(vname, decl);
-	    if (! var.isSimple())
-	      var = var.nextVar();  // Skip xxIncoming fake fields.
-	  }
-        PrimProcedure pproc = new PrimProcedure(method, lexp);
-        applyMethods.addElement(pproc);
-        applyMethodsCount[lexp.min_args <= 4 ? lexp.min_args : 5]++;
-	body = new ApplyExp(pproc, args);
-	lexp.heapFrame = null;
-      }
+    Variable heapFrame = lexp.heapFrame;
 
     Method apply_method;
     if (lexp.isModuleBody())
-      apply_method
-	  = curClass.addMethod ("run", arg_types, Type.pointer_type,
+      {
+	Type rtype = Type.pointer_type;
+	if (curClass.getSuperclass() != typeModuleBody)
+	  {
+	    curClass.addInterface(typeRunnable);
+	    rtype = Type.void_type;
+	  }
+	apply_method
+	  = curClass.addMethod ("run", arg_types, rtype,
 				Access.PUBLIC|Access.FINAL);
+      }
     else if (lexp.isHandlingTailCalls())
       apply_method
 	= curClass.addMethod ("apply", arg_types, Type.void_type,
@@ -777,48 +1084,21 @@ public class Compilation
 
     if (lexp.isHandlingTailCalls())
       {
-	Declaration decl = new Declaration ("stack", typeCallStack);
+	callStackContext = new Variable ("stack", typeCallStack);
 	Scope scope = lexp.scope;
-	scope.addVariableAfter(thisDecl, decl);
-	decl.setParameter(true);
-	decl.setArtificial(true);
-	callStackContext = decl;
-      }
-
-    Variable argsArray;
-    if (lexp.isHandlingTailCalls() || lexp.min_args != lexp.max_args || lexp.min_args > 4
-	|| (fewerClasses && curClass == mainClass))
-      argsArray = lexp.declareArgsArray();
-    else
-      argsArray = null;
-
-    if (usingCPStyle() || (fewerClasses && curClass == mainClass))
-      {
-	code = getCode();
-	if (lexp.isHandlingTailCalls())
-	  {
-	    fswitchIndex = pcCallStackField;
-	    callStackContext.reserveLocal(1, code);
-	    code.emitLoad(callStackContext);
-	  }
-	else
-	  {
-	    fswitchIndex = curClass.addField("PC", Type.int_type);
-	    code.emitPushThis();
-	  }
-	code.emitGetField(fswitchIndex);
-	fswitch = new SwitchState(code);
-	fswitch.addCase(0, code);
-	this.argsArray = argsArray;
+	scope.addVariableAfter(thisDecl, callStackContext);
+	callStackContext.setParameter(true);
+	callStackContext.setArtificial(true);
       }
 
     int line = lexp.getLine();
     if (line > 0)
       code.putLineNumber(line);
 
+    /*
     if (arg_letter == 'N')
       {
-	argsArray.reserveLocal(1, code);
+	argsArray.reserveLocal(1, code); // FIXME
 
 	if (true) // If generating code to check number of arguments
 	  {
@@ -828,14 +1108,25 @@ public class Compilation
 	    code.emitInvokeStatic(checkArgCountMethod);
 	  }
       }
+    */
 
-    lexp.allocParameters(this, argsArray);
-    lexp.enterFunction(this, argsArray);
+    lexp.allocParameters(this);
+    lexp.enterFunction(this);
 
-    body.compileWithPosition(this,
-			     lexp.isModuleBody () ? Target.pushObject
-			     : Target.returnObject);
+    try
+      {
+        body.compileWithPosition(this,
+                                 lexp.isModuleBody () ? Target.pushObject
+                                 : Target.returnObject);
+      }
+    catch (Exception ex)
+      {
+        error('f', "internal error while compiling - caught: "+ex);
+        ex.printStackTrace(System.err);
+        System.exit(-1);
+      }
     lexp.compileEnd(this);
+
     if (Compilation.fewerClasses) // FIXME
       method.popScope(); // Undoes pushScope in method.initCode.
 
@@ -848,116 +1139,29 @@ public class Compilation
       }
 
     if (lexp.isModuleBody ())
-      { // Generate ModuleBody's apply0 .. applyN methods:
-	Method save_method = method;
-        for (int i = 0;  i < 6; i++)
-          {
-            if (applyMethodsCount[i] == 0)
-              continue;
-            String mname = i < 5 ? ("apply"+i) : "applyN";
-            method = curClass.addMethod (mname, applyArgsModule[i],
-                                         Type.pointer_type,
-                                         Access.PUBLIC);
-            method.init_param_slots();
-            code = getCode();
-            int nargs;
-            code.emitLoad(code.getArg(1));
-            code.emitGetField(selectorOfModuleMethod);
-            SwitchState aswitch = new SwitchState(code);
-            for (int j = applyMethods.size();  --j >= 0; )
-              {
-                PrimProcedure apply
-                  = (PrimProcedure) applyMethods.elementAt(j);
-                LambdaExp source = apply.source;
-                boolean varArgs = source.min_args != source.max_args;
-                if (i != ((varArgs || source.min_args >= 5) ? 5
-                          : source.min_args))
-                  continue;
-                aswitch.addCase(source.getSelectorValue(this), code);
-                int sourceLine = source.getLine();
-                if (sourceLine > 0)
-                  code.putLineNumber(sourceLine);
-                Method primMethod = apply.method;
-                nargs = primMethod.getParameterTypes().length;
-                int needsThis = apply.getStaticFlag() ? 0 : 1;
-                if (needsThis > 0)
-                  code.emitPushThis();
-                if (varArgs)
-                  {
-                    code.emitLoad(code.getArg(2));
-                  }
-                else if (! varArgs)
-                  {
-                    Variable pvar = source.firstVar();
-                    int k = 0;
-                    for (; k < source.min_args;  pvar = pvar.nextVar())
-                      {
-                        if (pvar.isParameter () && ! pvar.isArtificial ())
-                          {
-                            sourceLine = ((Declaration) pvar).getLine();
-                            if (sourceLine > 0)
-                              code.putLineNumber(sourceLine);
-                            if (i > 4) // more than 4 args
-                              {
-                                // Load Object[]args value:
-                                code.emitLoad(code.getArg(2));
-                                code.emitPushInt(k);
-                                code.emitArrayLoad(Type.pointer_type);
-                              }
-                            else
-                              code.emitLoad(code.getArg(k + 2));
-                            Type ptype = primMethod.getParameterTypes()[k];
-                            if (ptype != Type.pointer_type)
-                              CheckedTarget.emitCheckedCoerce(this, source,
-                                                              k, ptype);
-                            k++;
-                          }
-                      }
-                  }
-                code.emitInvokeMethod(apply.method, apply.op_code); 
-                Target.pushObject.compileFromStack(this, apply.getReturnType());
-                code.emitReturn();
-              }
-            aswitch.addDefault(code);
-            nargs = method.getParameterTypes().length;
-            if (! method.getStaticFlag())
-              nargs++;
-            for (int k = 0;  k < nargs;  k++)
-              code.emitLoad(code.getArg(k));
-            code.emitInvokeSpecial(applyModuleMethods[i]);  // call super
-            code.emitReturn();
-            aswitch.finish(code);
-          }
-	method = save_method;
-      }
+      generateApplyMethods();
 
-    if (curClass == mainClass
-	&& ((! immediate && literalsChain != null)
-	    || bindingFields.size() > 0))
+    if (curClass == mainClass && ! immediate && clinitChain != null)
       {
 	Method save_method = method;
 	method = curClass.addMethod ("<clinit>", apply0args, Type.void_type,
 				     Access.PUBLIC|Access.STATIC);
 	method.init_param_slots ();
-	if (! immediate)
-	  dumpLiterals ();
+
 	code = getCode();
-	int numGlobals = bindingFields.size();
-	if (numGlobals > 0)
+
+	if (generateMain || generateApplet)
 	  {
-	    code.emitInvokeStatic(getCurrentEnvironmentMethod);
-	    java.util.Enumeration e = bindingFields.keys();
-	    while (e.hasMoreElements())
-	      {
-		String id = (String) e.nextElement();
-		Field fld = (Field) bindingFields.get(id);
-		if (--numGlobals > 0)
-		  code.emitDup(1);
-		code.emitPushString(id);
-		code.emitInvokeVirtual(getBindingEnvironmentMethod);
-		code.emitPutStatic(fld);
-	      }
+	    ClassType interpreterType
+	      = (ClassType) Type.make(getInterpreter().getClass());
+	    Method registerMethod
+	      = interpreterType.getDeclaredMethod("registerEnvironment", 0);
+	    if (registerMethod != null)
+	      code.emitInvokeStatic(registerMethod);
 	  }
+
+	dumpInitializers(clinitChain);
+
 	code.emitReturn();
 	method = save_method;
       }
@@ -1014,7 +1218,9 @@ public class Compilation
     if (fld != null)
       return (Field) fld;
     String fieldName = "id"+bindingFields.size()+"$"+mangleName(name);
-    Field field = mainClass.addField(fieldName, typeBinding, Access.STATIC);
+    Type ftype = getInterpreter().hasSeparateFunctionNamespace()
+      ? typeBinding2 : typeBinding;
+    Field field = mainClass.addField(fieldName, ftype, Access.STATIC);
     bindingFields.put(name, field);
     return field;
   }
