@@ -38,17 +38,14 @@ public abstract class Expression implements Printable
   {
     String saveFilename = comp.filename;
     int savePosition = comp.position;
-    try
-      {
-        comp.filename = filename;
-        comp.position = position;
-        compile(comp, target);
-      }
-    finally
-      {
-        comp.filename = saveFilename;
-        comp.position = savePosition;
-      }
+    comp.filename = filename;
+    comp.position = position;
+    compile(comp, target);
+    // This might logically belong in a `finally' clause.
+    // It is intentionally not so, so if there is an internal error causing
+    // an exception, we get the line number where the exception was thrown.
+    comp.filename = saveFilename;
+    comp.position = savePosition;
   }
 
   public final void compile (Compilation comp, Type type)
@@ -62,6 +59,25 @@ public abstract class Expression implements Printable
   String filename;
   int position;
 
+  public static final Expression[] noExpressions = new Expression[0];
+
+  /** Helper method to create a `while' statement. */
+  public static Expression makeWhile(Expression cond, Expression body)
+  {
+    Expression[] inits = new Expression[1];
+    LetExp let = new LetExp(inits);
+    Declaration fdecl = let.addDeclaration("%do%loop");
+    Expression recurse = new ApplyExp(new ReferenceExp(fdecl), noExpressions);
+    IfExp lbody = new IfExp(cond,
+			    new BeginExp(body, recurse),
+			    QuoteExp.voidExp);
+    LambdaExp lexp = new LambdaExp(lbody);
+    inits[0] = lexp;
+    fdecl.noteValue(lexp);
+    let.setBody(new ApplyExp(new ReferenceExp(fdecl), noExpressions));
+    return let;
+  }
+
   public final void setFile (String filename)
   {
     this.filename = filename;
@@ -100,4 +116,29 @@ public abstract class Expression implements Printable
     return Type.pointer_type;
   }
 
+  protected int flags;
+  protected static final int NEXT_AVAIL_FLAG = 1;
+
+  public void setFlag (boolean setting, int flag)
+  {
+    if (setting) flags |= flag;
+    else flags &= ~flag;
+  }
+
+  public void setFlag (int flag)
+  {
+    flags |= flag;
+  }
+
+  public int  getFlags()
+  {
+    return flags;
+  }
+
+  public boolean getFlag (int flag)
+  {
+    return (flags & flag) != 0;
+  }
+
+
 }
