@@ -16,6 +16,8 @@ public class QuoteExp extends Expression
   {
     if (value == Values.empty)
       return gnu.bytecode.Type.void_type;
+    if (value == null)
+      return gnu.bytecode.Type.pointer_type;
     return gnu.bytecode.Type.make(value.getClass());
   }
 
