@@ -24,11 +24,32 @@ public class ReferenceExp extends Expression
   /** Unique id number, to ease print-outs and debugging. */
   int id = ++counter;
 
-  boolean dontDereference;
+  private static int DONT_DEREFERENCE = NEXT_AVAIL_FLAG;
+  private static int PROCEDURE_NAME = NEXT_AVAIL_FLAG << 1;
+  public static int PREFER_BINDING2 = NEXT_AVAIL_FLAG << 2;
+
   /* If true, must have binding.isBinding().  Don't dereference Binding. */
-  public final boolean getDontDereference() { return  dontDereference; }
-  public final void setDontDereference(boolean dontDereference)
-  { this.dontDereference = dontDereference; }
+  public final boolean getDontDereference()
+  {
+    return (flags & DONT_DEREFERENCE) != 0;
+  }
+
+  public final void setDontDereference(boolean setting)
+  { setFlag(setting, DONT_DEREFERENCE); }
+
+  /** True if this identifier appears in "function call position".
+   * If so, it should be interpreted as a function name, which makes a
+   * difference for languages (like Common Lisp) that have two name spaces. */
+  public final boolean isProcedureName()
+  {
+    return (flags & PROCEDURE_NAME) != 0;
+  }
+
+  /** Note if this identifier appears in "function call position". */
+  public final void setProcedureName(boolean setting)
+  {
+    setFlag(setting, PROCEDURE_NAME);
+  }
 
   public ReferenceExp (String symbol)
   {
@@ -52,41 +73,51 @@ public class ReferenceExp extends Expression
     if (binding != null
         && ! (binding.context instanceof ModuleExp && ! binding.isPrivate()))
       throw new Error("internal error: ReferenceExp.eval on lexical binding");
-    return env.getChecked(symbol);
+    if (getFlag(PREFER_BINDING2))
+      {
+	Binding bind = Binding2.getBinding2(env, symbol);
+	return isProcedureName() ? bind.getProcedure() : bind.get();
+      }
+    else
+      return env.getChecked(symbol);
   }
 
-  private static ClassType thisType;
-  static ClassType typeLocation = null;
-  static Method getMethod = null;
-
   public void compile (Compilation comp, Target target)
   {
     if (target instanceof IgnoreTarget)
       return;
     CodeAttr code = comp.getCode();
-    if (binding != null
-        && ! (binding.context instanceof ModuleExp && ! binding.isPrivate()))
+    if (binding != null)
       {
 	binding.load(comp);
 	if (binding.isIndirectBinding() && ! getDontDereference())
 	  {
-	    if (typeLocation == null)
-	      {
-		typeLocation = ClassType.make("gnu.mapping.Location");
-		getMethod = typeLocation.addMethod("get",
-						   Type.typeArray0,
-						   Compilation.typeObject,
-						   Access.PUBLIC|Access.FINAL);
-	      }
-	    code.emitInvokeVirtual (getMethod);
+	    code.emitInvokeVirtual (Compilation.getLocationMethod);
 	  }
 	else if (binding.isFluid())
 	  code.emitGetField(FluidLetExp.valueField);
       }
     else
       {
-	code.emitGetStatic(comp.getBindingField(symbol));
-	code.emitInvokeVirtual(Compilation.getBindingMethod);
+	Field field = comp.getBindingField(symbol);
+	if (field.getStaticFlag())
+	  code.emitGetStatic(field);
+	else
+	  {
+	    LambdaExp lexp = comp.curLambda;
+	    while (! (lexp instanceof ModuleExp))
+	      lexp = lexp.outerLambda();
+	    lexp.loadHeapFrame(comp);
+	    code.emitGetField(field);
+	  }
+	if (! isProcedureName())
+	  code.emitInvokeVirtual(Compilation.getLocationMethod);
+	else //if (! comp.getInterpreter().hasSeparateFunctionNamespace())
+	  code.emitInvokeVirtual(Compilation.getProcedureBindingMethod);
+	/*
+	else
+	  code.emitGetField(Compilation.functionValueBinding2Field);
+	*/
       }
     target.compileFromStack(comp, getType());
   }
