@@ -3,7 +3,7 @@ import gnu.bytecode.Type;
 import gnu.bytecode.ClassType;
 import gnu.mapping.*;
 import gnu.expr.*;
-import kawa.lang.FString;
+import gnu.kawa.util.FString;
 
 public class ClassMethods extends ProcedureN
 {
@@ -14,10 +14,10 @@ public class ClassMethods extends ProcedureN
     return apply(this, args[0], args[1], null, null, 0, 0);
   }
 
-  public static Procedure apply(Procedure thisProc,
-                                Object arg0, Object arg1,
-                                Type rtype, Type[] atypes,
-                                int modifiers, int modmask)
+  public static MethodProc apply(Procedure thisProc,
+                                 Object arg0, Object arg1,
+                                 Type rtype, Type[] atypes,
+                                 int modifiers, int modmask)
   {
     ClassType dtype;
     String mname;
@@ -33,15 +33,22 @@ public class ClassMethods extends ProcedureN
       mname = arg1.toString();
     else
       throw new WrongType(thisProc, 1, null);
-    mname = Compilation.mangleName(mname);
-    return apply(dtype, mname, null, null, 0, 0);
+    if (! ("<init>".equals(mname)))
+      mname = Compilation.mangleName(mname);
+    MethodProc result = apply(dtype, mname, rtype, atypes, modifiers, modmask);
+    if (result == null)
+      throw new RuntimeException("no applicable method named `"+mname+"' in "
+                                 +dtype.getName());
+    return result;
   }
 
-  public static MethodProc apply(ClassType dtype, String mname,
-                                 Type rtype, Type[] atypes,
-                                 int modifiers, int modmask)
+  /** Return the methods of a class wit the specified name and flag.
+   * @return an array containing the methods.
+   */
+  public static PrimProcedure[] getMethods(ClassType dtype, String mname,
+                                           int modifiers, int modmask,
+                                           Interpreter interpreter)
   {
-    kawa.lang.Interpreter interpreter = kawa.standard.Scheme.getInstance();
     Class dclass = dtype.getReflectClass();
     if (dclass == null)
       throw new RuntimeException("no such class: "+dtype.getName());
@@ -53,9 +60,11 @@ public class ClassMethods extends ProcedureN
       methods = getMethods(dclass, mname, modifiers, modmask);
     GenericProc gproc = null;
     PrimProcedure pproc = null;
+    PrimProcedure[] result = new PrimProcedure[methods.length];
     for (int i = 0;  i < methods.length;  i++)
       {
         gnu.bytecode.Method method;
+        PrimProcedure cur;
         if (wantConstructor)
           {
             java.lang.reflect.Constructor rmethod
@@ -68,6 +77,8 @@ public class ClassMethods extends ProcedureN
             method
               = dtype.addMethod("<init>", rmethod.getModifiers(), params,
                                 Type.void_type);
+            cur = new PrimProcedure(method);
+            cur.setReturnType(dtype);
           }
         else
           {
@@ -82,13 +93,90 @@ public class ClassMethods extends ProcedureN
               = dtype.addMethod(rmethod.getName(),
                                 rmethod.getModifiers(), params,
                                 interpreter.getTypeFor(rmethod.getReturnType()));
+            cur = new PrimProcedure(method);
+          }
+        result[i] = cur;
+      }
+    return result;
+  }
+
+  /** Re-order the methods such that the ones that are definite
+   * applicable (all argtypes is subset of parameter type) are first;
+   * those possibly applicable next (argtype overlaps parameter types);
+   * and ending with those definitely not applicable (some argtype does
+   * overlap its parameter type).
+   * @return ((number of definitely applicabable methods) << 32
+   *          + (number of possibly applicable methods.
+   */
+  public static long selectApplicable(PrimProcedure[] methods,
+                                      Type[] atypes)
+  {
+    int limit = methods.length;
+    int numDefApplicable = 0;
+    int numPosApplicable = 0;
+    for (int i = 0;  i < limit;  )
+      {
+        int code = methods[i].isApplicable(atypes);
+        if (code < 0)
+          { // Definitely not applicable.
+            // swap(methods[limit-1], methods[i]):
+            PrimProcedure tmp = methods[limit-1];
+            methods[limit-1] = methods[i];
+            methods[i] = tmp;
+            limit--;
+          }
+        else if (code > 0)
+          { // Definitely applicable.
+            // swap(methods[numDefApplicable], methods[i]):
+            PrimProcedure tmp = methods[numDefApplicable];
+            methods[numDefApplicable] = methods[i];
+            methods[i] = tmp;
+            numDefApplicable++;
+            i++;
+          }
+        else
+          { // Possibly applicable.
+            numPosApplicable++;
+            i++;
+          }
+      }
+    return (((long) numDefApplicable) << 32) + (long) numPosApplicable;
+  }
+
+  public static MethodProc apply(ClassType dtype, String mname,
+                                 Type rtype, Type[] atypes,
+                                 int modifiers, int modmask)
+  {
+    Interpreter interpreter = Interpreter.defaultInterpreter; // FIXME
+    PrimProcedure[] methods = getMethods(dtype, mname,
+                                         modifiers, modmask, interpreter);
+    GenericProc gproc = null;
+    PrimProcedure pproc = null;
+    for (int i = 0;  i < methods.length;  i++)
+      {
+        PrimProcedure cur = methods[i];;
+        if (atypes != null)
+          {
+            int applicable = cur.isApplicable(atypes);
+            if (applicable == -1)
+              continue;
+            if (pproc != null)
+              {
+                MethodProc best = MethodProc.mostSpecific(pproc, cur);
+                if (best != null)
+                  {
+                    if (cur == best)
+                      pproc = cur;
+                    continue;
+                  }
+              }
           }
         if (pproc != null && gproc == null)
           {
             gproc = new GenericProc();
             gproc.add(pproc);
           }
-        pproc = new PrimProcedure(method);
+        pproc = cur;
         if (gproc != null)
           gproc.add(pproc);
       }
@@ -97,10 +185,7 @@ public class ClassMethods extends ProcedureN
         gproc.setName(dtype.getName()+"."+mname);
         return gproc;
       }
-    if (pproc != null)
-      return pproc;
-    throw new RuntimeException("no method named `"+mname+"' in "
-                               +dtype.getName());
+    return pproc;
   }
 
   /** Re-order method array in place so one with selected name are first.
@@ -147,9 +232,22 @@ public class ClassMethods extends ProcedureN
   {
     java.lang.reflect.Constructor[] methods = clas.getConstructors();
     int nmethods = selectMethods(methods, null, modifiers, modmask);
+    if (nmethods == methods.length)
+      return methods;
     java.lang.reflect.Constructor[] matching
     = new java.lang.reflect.Constructor[nmethods];
     System.arraycopy(methods, 0, matching, 0, nmethods);
     return matching;
   }
+
+  static String checkName(Expression exp)
+  {
+    if (exp instanceof QuoteExp)
+      {
+        Object name = ((QuoteExp) exp).getValue();
+        if (name instanceof FString || name instanceof String)
+          return Compilation.mangleName(name.toString());
+      }
+    return null;
+  }
 }
