@@ -219,8 +219,10 @@ class Break
  * until more general output framework (e.g. pretty-printing).
  */
 
-class BufferedPort extends FilterWriter
+class BufferedPort extends Writer
 {
+  protected Writer out;
+
   char[] buffer;
 
   /** First used (unflushed) position in buffer. */
@@ -234,7 +236,7 @@ class BufferedPort extends FilterWriter
 
   BufferedPort(Writer out, int bufsize)
   {
-    super(out);
+    this.out = out;
     buffer = new char[bufsize];
   }
 
@@ -290,6 +292,13 @@ class BufferedPort extends FilterWriter
   void flushLocal()
     throws IOException
   {
+    if (out == null)
+      {
+        char[] newBuffer = new char[2 * buffer.length];
+        System.arraycopy(buffer, 0, newBuffer, 0, bufWritePos);
+        buffer = newBuffer;
+        return;
+      }
     int i = bufWritePos;
     for (;;)
       {
@@ -313,6 +322,8 @@ class BufferedPort extends FilterWriter
   public void flush()
     throws IOException
   {
+    if (out == null)
+      return;
     flushLocal();
     out.flush();
   }
@@ -333,7 +344,12 @@ class BufferedPort extends FilterWriter
   public void close()
     throws IOException
   {
-    flushLocal();
-    out.close();
+    if (out != null)
+      {
+        flushLocal();
+        out.close();
+        out = null;
+      }
+    buffer = null;
   }
 }
