@@ -5,10 +5,10 @@ import java.awt.event.*;
 import java.io.*;
 import gnu.mapping.*;
 
+import gnu.expr.Interpreter;
 import kawa.standard.Scheme;
-import kawa.lang.Interpreter;
 
-/** A Frame containing a Kwa read-eval-print loop.
+/** A Frame containing a Kawa read-eval-print loop.
   * @author Albert Ting <alt@artisan.com> (original base)
   * @author Per Bothner (extensive changes).
   */
@@ -28,7 +28,7 @@ public class GuiConsole extends Frame implements ActionListener {
   Environment environment;
   Future thread;
 
-  kawa.lang.QueueReader in_r;
+  gnu.text.QueueReader in_r;
   OutPort out_p, err_p;
 
   MessageArea message = null;
@@ -38,12 +38,17 @@ public class GuiConsole extends Frame implements ActionListener {
     new GuiConsole(interp, interp.getNewEnvironment());
   }
 
+  public GuiConsole()
+  {
+    this(Interpreter.defaultInterpreter, Environment.getCurrent());
+  }
+
   public GuiConsole(Interpreter interp, Environment environment) {
     super("Kawa");
     this.interp = interp;
     this.environment = environment;
 
-    in_r = new kawa.lang.QueueReader ();
+    in_r = new gnu.text.QueueReader ();
     message = new MessageArea(false, in_r);
     window_number++;
     numConsoles++;
