@@ -5,8 +5,8 @@ import kawa.standard.*;
 import gnu.mapping.*;
 import gnu.expr.*;
 import java.io.*;
-import gnu.text.SourceMessages;
-import gnu.text.Lexer;
+import gnu.text.*;
+import gnu.kawa.util.*;
 
 /** Utility functions (static methods) for kawa.repl.
  * Should probably be merged with kawa.repl.  FIXME. */
@@ -23,9 +23,12 @@ public class Shell
     InPort inp = InPort.inDefault ();
     if (inp instanceof TtyInPort)
       {
-	Object prompter = env.get("default-prompter");
-	if (prompter != null && prompter instanceof Procedure)
-	  ((TtyInPort)inp).setPrompter((Procedure) prompter);
+	Binding pr = Environment.getCurrentBinding("default-prompter");
+	if (pr != null) // FIXME - is this how we want it to work?
+	  {
+	    Procedure prompter = pr.getProcedure();
+	    ((TtyInPort)inp).setPrompter(prompter);
+	  }
       }
 
     run(interp, env, inp, OutPort.outDefault(), OutPort.errDefault());
@@ -36,6 +39,7 @@ public class Shell
   {
     SourceMessages messages = new SourceMessages();
     Translator tr = new Translator(env, messages);
+    tr.immediate = true;
     Lexer lexer = interp.getLexer(inp, messages);
     for (;;)
       {
@@ -43,7 +47,7 @@ public class Shell
 	  {
 	    lexer.clearErrors();
 	    PairWithPosition body = new PairWithPosition(inp,
-							 null, List.Empty);
+							 null, LList.Empty);
 	    Object sexp = ((gnu.text.LispReader) lexer).readObject(); // FIXME
 
             // Skip whitespace, in case somebody calls (read-char) or similar.
@@ -116,11 +120,11 @@ public class Shell
 	if (fname.equals ("-"))
 	  kawa.standard.load.loadSource(InPort.inDefault(), env);
 	else
-	  {
-	    InPort fstream = InPort.openFile(fname);
-	    kawa.standard.load.loadSource(fstream, env);
-	    fstream.close();
-	  }
+	  kawa.standard.load.apply(fname,env);
+      }
+    catch (gnu.text.SyntaxException e)
+      {
+	e.printAll(OutPort.errDefault(), 20);
       }
     catch (FileNotFoundException e)
       {
