@@ -1,6 +1,7 @@
 package kawa.lang;
 import gnu.mapping.*;
 import gnu.expr.*;
+import gnu.kawa.util.*;
 
 /**
  * Implement autoloading of Syntax (including macros).
@@ -61,9 +62,14 @@ public class AutoloadSyntax extends Syntax
 	Object value = Class.forName (className).newInstance ();
 	if (value instanceof ModuleBody)
 	  {
-            Environment env = Environment.current ();
+	    Environment env = Environment.current ();
+	    gnu.kawa.reflect.ClassMemberConstraint.defineAll(value, env);
 	    ((ModuleBody) value).run();
-	    value = env.get (name);
+	    // FIXME Kludge for ELisp.
+	    if (gnu.expr.Interpreter.getInterpreter().hasSeparateFunctionNamespace())
+	      value = gnu.jemacs.lang.Symbol.getFunctionBinding(env, name);
+	    else
+	      value = env.get (name);
 	    if (value == null || value == this
 		|| !(value instanceof Syntax))
 	      throw_error("syntax not found in ");
@@ -95,6 +101,24 @@ public class AutoloadSyntax extends Syntax
       { throw_error ("type error"); }
   }
 
+  public boolean scanForDefinitions (Pair st, java.util.Vector forms,
+                                     ScopeExp defs, Translator tr)
+  {
+    if (loaded == null)
+      {
+	try
+	  {
+	    load ();
+	  }
+	catch (RuntimeException e)
+	  {
+	    tr.syntaxError (e.getMessage ());
+            return false;
+	  }
+      }
+    return loaded.scanForDefinitions(st, forms, defs, tr);
+  }
+
   public Expression rewriteForm (Pair form, Translator tr)
   {
     if (loaded == null)
