@@ -1,5 +1,6 @@
 package kawa.lang;
-import gnu.bytecode.ClassType;
+import gnu.bytecode.*;
+import gnu.expr.Compilation;
 
 /**
  * A Pattern is used to match against objects.
@@ -29,4 +30,10 @@ abstract public class Pattern
   abstract public boolean match (Object obj, Object[] vars, int start_vars);
 
   abstract public int varCount ();
+
+  static public ClassType typePattern = ClassType.make("kawa.lang.Pattern");
+  private static Type[] matchArgs =
+  { Type.pointer_type, Compilation.objArrayType, Type.int_type };
+  static public final Method matchPatternMethod
+  = typePattern.addMethod("match", matchArgs, Type.boolean_type, Access.PUBLIC);
 }
