@@ -1,6 +1,7 @@
 package kawa.lang;
 import gnu.mapping.*;
 import gnu.expr.*;
+import gnu.kawa.util.*;
 
 /**
  * Abstract class for "syntax" objects.
@@ -8,18 +9,16 @@ import gnu.expr.*;
  * @author	Per Bothner
  */
 
-abstract public class Syntax extends Declaration implements Printable
+abstract public class Syntax extends Declaration implements Printable, Named
 {
   public Syntax ()
   {
-    super ();
     setSimple(false);
   }
 
   public Syntax (String name)
   {
     setName(name);
-    sym = name;
     setSimple(false);
   }
 
@@ -30,6 +29,7 @@ abstract public class Syntax extends Declaration implements Printable
    * @param tr the Translator that provides context
    * @return the re-written expression
    */
+
   public Expression rewrite (Object obj, Translator tr)
   {
     throw new InternalError("rewrite method not defined");
@@ -40,6 +40,20 @@ abstract public class Syntax extends Declaration implements Printable
     return rewrite(form.cdr, tr);
   }
 
+  /** Check if a statement is a definition, for initial pass.
+   * @param st the statement to check
+   * @param form where to append the (possibly-modified) statement
+   * @param defs where to add Declarations for found definitions
+   * @param tr the compilation state
+   * @return true on success
+   */
+  public boolean scanForDefinitions (Pair st, java.util.Vector forms,
+                                    ScopeExp defs, Translator tr)
+  {
+    forms.addElement(st);
+    return true;
+  }
+
   public void print(java.io.PrintWriter ps)
   {
     ps.print("#<syntax ");
