@@ -6,6 +6,7 @@ import gnu.bytecode.Type;
 import gnu.bytecode.ArrayType;
 import gnu.mapping.*;
 import gnu.expr.*;
+import gnu.kawa.util.*;
 
 public class SyntaxRule implements Compilable
 {
@@ -70,7 +71,7 @@ public class SyntaxRule implements Compilable
     template_identifiers.copyInto (this.template_identifiers);
     this.captured_decls = new Object[num_identifiers];
     for (int i = num_identifiers;  --i >= 0; )
-      captured_decls[i] = tr.current_decls.get(this.template_identifiers[i]);
+      captured_decls[i] = tr.environ.get(this.template_identifiers[i]);
   }
 
   static final String dots3 = "...";
@@ -174,7 +175,7 @@ public class SyntaxRule implements Compilable
 	      }
 	  }
 	int code;
-	if (pair.cdr == List.Empty)
+	if (pair.cdr == LList.Empty)
 	  code = LIST1;
 	else
 	  {
@@ -265,8 +266,7 @@ public class SyntaxRule implements Compilable
 	String renamed_symbol = Symbol.makeUninterned (name);
 	vars[num_variables + i] = renamed_symbol;
         Object captured = captured_decls == null ? null : captured_decls[i];
-	tr.current_decls.put (renamed_symbol,
-                              captured == null ? name : captured);
+	tr.environ.put(renamed_symbol, captured == null ? name : captured);
       }
     return execute_template (0, vars, 0, indexes, tr, form);
   }
@@ -308,7 +308,7 @@ public class SyntaxRule implements Compilable
 	else if (ch == LIST1)
 	  {
 	    Object car = stack.pop ();
-	    stack.push (new Pair (car, List.Empty));
+	    stack.push (new Pair (car, LList.Empty));
 	  }
 	else if (ch == START_REPEAT)
 	  {
@@ -355,7 +355,7 @@ public class SyntaxRule implements Compilable
 	      }
 	    Pair last = null;
 	    Object following = stack.pop ();
-	    Object result = List.Empty;
+	    Object result = LList.Empty;
 	    for (int j = 0;  j < count; j++)
 	      {
 		indexes[nesting] = j;
@@ -363,7 +363,7 @@ public class SyntaxRule implements Compilable
 						   vars,
 						   nesting + 1, indexes,
 						   tr, form);
-		Pair pair = new Pair (element, List.Empty);
+		Pair pair = new Pair (element, LList.Empty);
 		if (last == null)
 		  result = pair;
 		else
