@@ -8,6 +8,7 @@ import java.lang.reflect.Modifier;
 import gnu.bytecode.Type;
 import gnu.bytecode.ClassType;
 import gnu.text.SourceMessages;
+import gnu.kawa.util.*;
 
 /** Used to translate from source to Expression.
  * The result has macros expanded, lexical names bound, etc, and is
@@ -20,16 +21,18 @@ import gnu.text.SourceMessages;
 public class Translator extends Object
 {
   // Map name to Declaration.
-  public Environment current_decls;
+  public Environment environ;
   ScopeExp current_scope;
 
-  // This is null in JDK 1.1 and something else in JDK 1.2.
-  private ClassLoader systemClassLoader = getClass().getClassLoader();
+  /** If doing immediate evaluation. */
+  public boolean immediate;
 
   public LambdaExp currentLambda () { return current_scope.currentLambda (); }
 
   public ScopeExp currentScope() { return current_scope; }
 
+  public Interpreter getInterpreter() { return Interpreter.getInterpreter(); }
+
   /** Return true if decl is lexical and not fluid. */
   public boolean isLexical (Declaration decl)
   {
@@ -62,14 +65,14 @@ public class Translator extends Object
   public Translator (Environment env, SourceMessages messages)
   {
     this.env = env;
-    current_decls = new Environment();
+    environ = new Environment();
     this.messages = messages;
   }
 
   public Translator (Environment env)
   {
     this.env = env;
-    current_decls = new Environment();
+    environ = new Environment();
     messages = new SourceMessages();
   }
 
@@ -109,7 +112,7 @@ public class Translator extends Object
       }
     finally
       {
-        currentSyntax = currentSyntax;
+        currentSyntax = saveSyntax;
       }
     return exp;
   }
@@ -163,11 +166,21 @@ public class Translator extends Object
     if (obj instanceof String)
       {
 	String sym = (String) obj;
-	obj = current_decls.get (sym);
+	Binding binding = environ.lookup (sym);
+	if (binding == null)
+	  obj = null;
+	else
+	  obj = binding.getFunctionValue();
 
         if (obj instanceof Syntax)
           return obj;
-	Binding binding = null;
+	if (obj instanceof Declaration)
+	  {
+	    Expression dval = ((Declaration) obj).getValue();
+	    if (dval instanceof QuoteExp)
+	      return ((QuoteExp) dval).getValue();
+	  }
+	binding = null;
         if (obj != null)
 	  {
 	    // Hygenic macro expansion may bind a renamed (uninterned) symbol
@@ -201,8 +214,9 @@ public class Translator extends Object
 
   public Expression rewrite_pair (Pair p)
   {
-    if (p.car instanceof Syntax)
-      return apply_rewrite((Syntax) p.car, p);
+    Syntax syn = check_if_Syntax(p.car);
+    if (syn != null)
+      return apply_rewrite(syn, p);
     Object cdr = p.cdr;
 
     Expression func = rewrite_car (p);
@@ -217,8 +231,8 @@ public class Translator extends Object
 	  {
             String name = ref.getName();
             Binding binding = env.lookup((String) name);
-	    if (binding != null && binding.isBound())
-              proc = binding.get();
+	    if (binding != null)
+	      proc = binding.getFunctionValue();
 	    if (proc instanceof Syntax)
 	      return apply_rewrite ((Syntax) proc, p);
             if (proc instanceof AutoloadProcedure)
@@ -239,7 +253,7 @@ public class Translator extends Object
           return apply_rewrite ((Syntax) decl, p);
       }
 
-    int cdr_length = List.length (cdr);
+    int cdr_length = LList.length (cdr);
 
     if (func instanceof QuoteExp)
       {
@@ -263,7 +277,8 @@ public class Translator extends Object
       }
 
   tryDirectCall:
-    if (proc != null && proc instanceof Procedure)
+    if (proc != null && proc instanceof Procedure
+        && ! (proc instanceof Inlineable))
       {
 	if (proc instanceof AutoloadProcedure)
 	  {
@@ -273,64 +288,26 @@ public class Translator extends Object
 	      }
 	    catch (RuntimeException ex)
 	      {
+                ex.printStackTrace(System.err);
 		break tryDirectCall;
 	      }
-	    if (proc == null || ! (proc instanceof Procedure))
+	    if (proc == null || ! (proc instanceof Procedure)
+                || proc instanceof Inlineable)
 	      break tryDirectCall;
 	  }
-	Class procClass = proc.getClass();
-	if (procClass.getClassLoader() != systemClassLoader)
-	  break tryDirectCall;
-	try
-	  {
-	    java.lang.reflect.Method[] meths = procClass.getDeclaredMethods();
-	    java.lang.reflect.Method best = null;
-	    Class[] bestTypes = null;
-            Procedure pproc = (Procedure) proc;
-	    String name = pproc.getName();
-	    if (name == null)
-	      break tryDirectCall;
-	    String mangledName = Compilation.mangleName(name);
-	    for (int i = meths.length;  --i >= 0; )
-	      {
-		java.lang.reflect.Method meth = meths[i];
-		int mods = meth.getModifiers();
-		if ((mods & (Modifier.STATIC|Modifier.PUBLIC))
-		    != (Modifier.STATIC|Modifier.PUBLIC))
-		  continue;
-		String mname = meth.getName();
-		if (! mname.equals("apply") && ! mname.equals(mangledName))
-		  continue;
-		Class[] ptypes = meth.getParameterTypes();
-		if (ptypes.length != cdr_length)
-		  continue;
-		// In the future, we may try to find the "best" match.
-		if (best != null)
-		  return syntaxError("ambiguous inline for call to "+name
-				     + " (in class "+procClass.getName()+")");
-		best = meth;
-		bestTypes = ptypes;
-	      }
-	    if (best != null)
-	      {
-		ClassType procType = ClassType.make(procClass.getName());
-		Type[] argTypes = new Type[bestTypes.length];
-		for (int i = argTypes.length;  --i >= 0; )
-		  argTypes[i] = Type.make(bestTypes[i]);
-		gnu.bytecode.Method method
-		  = procType.addMethod(best.getName(), best.getModifiers(),
-				       argTypes,
-				       Type.make(best.getReturnType()));
-                pproc = new PrimProcedure(method);
-                pproc.setName(name);
-		func = new QuoteExp(pproc);
-	      }
-	  }
-	catch (SecurityException ex)
-	  {
-	  }
+
+        PrimProcedure pproc
+          = PrimProcedure.getMethodFor((Procedure) proc, args);
+        if (pproc != null)
+          func = new QuoteExp(pproc);
       }
 
+    if (func instanceof ReferenceExp)
+      {
+	((ReferenceExp) func).setProcedureName(true);
+	if (getInterpreter().hasSeparateFunctionNamespace())
+	  func.setFlag(ReferenceExp.PREFER_BINDING2);
+      }
     return new ApplyExp (func, args);
   }
 
@@ -346,15 +323,22 @@ public class Translator extends Object
     else if (exp instanceof String)
       {
 	String name = (String) exp;
-	Object binding = current_decls.get (name);
+	Object binding = environ.get (name);
+	Declaration decl = null;
 	// Hygenic macro expansion may bind a renamed (uninterned) symbol
 	// to the original symbol.  Here, use the original symbol.
 	if (binding != null && binding instanceof String)
-	  return new ReferenceExp ((String) binding);
-	Declaration decl = (Declaration) binding;
-	if (! isLexical(decl))
-	  decl = null;
-	return new ReferenceExp (name, decl);
+	  name = (String) binding;
+        else if (binding instanceof Declaration) // ?? FIXME
+          {
+            decl = (Declaration) binding;
+            if (! isLexical(decl))
+              decl = null;
+          }
+	ReferenceExp rexp = new ReferenceExp (name, decl);
+	if (getInterpreter().hasSeparateFunctionNamespace())
+	  rexp.setFlag(ReferenceExp.PREFER_BINDING2);
+	return rexp;
       }
     else if (exp instanceof Expression)
       return (Expression) exp;
@@ -396,151 +380,42 @@ public class Translator extends Object
 
   boolean scan_form (Object st, java.util.Vector forms, ScopeExp defs)
   {
-    for (;;)
+    // Process st.
+    if (! (st instanceof Pair))
+      forms.addElement (st);
+    else
       {
-        // Process st.
-        if (! (st instanceof Pair))
-          forms.addElement (st);
-        else
-          {
-            Pair st_pair = (Pair) st;
-            Object op = st_pair.car;
-            Syntax syntax = check_if_Syntax (op);
-
-            if (syntax != null && syntax instanceof Macro)
-              {
-                String save_filename = current_filename;
-                int save_line = current_line;
-                int save_column = current_column;
-                Syntax saveSyntax = currentSyntax;
-                try
-                  {
-                    if (st_pair instanceof PairWithPosition)
-                      {
-                        PairWithPosition ppair = (PairWithPosition) st_pair;
-                        current_filename = ppair.getFile ();
-                        current_line = ppair.getLine ();
-                        current_column = ppair.getColumn ();
-                      }
-                    currentSyntax = syntax;
-                    st = ((Macro) syntax).expand (st_pair, this);
-                  }
-                finally
-                  {
-                    current_filename = save_filename;
-                    current_line = save_line;
-                    current_column = save_column;
-                    currentSyntax = saveSyntax;
-                  }
-                continue;
-		}
-            else if (syntax == Scheme.beginSyntax)
-              {
-                if (! scan_body (st_pair.cdr, forms, defs))
-                  return false;
-              }
-            /*
-            else if (syntax == Scheme.defineSyntax
-                     && st_pair.cdr instanceof Pair
-                     && ! (current_scope instanceof ModuleExp))
-              {
-                Object name = ((Pair) st_pair.cdr).car;
-                if (name instanceof String)
-                  defs.addDeclaration((String) name);
-                else if (name instanceof Pair)
-                  {
-                    Pair name_pair = (Pair) name;
-                    if (name_pair.car instanceof String)
-                      defs.addDeclaration((String) name_pair.car);
-                  }
-                forms.addElement (st);
-              }
-            */
-            else if ((syntax == Scheme.defineSyntax
-                      || syntax == Scheme.defineSyntaxPrivate)
-                     // Later:  || syntax == Scheme.defineSyntax
-                     && st_pair.cdr instanceof Pair)
-              {
-                boolean makePrivate = syntax == Scheme.defineSyntaxPrivate;
-                Pair p = (Pair) st_pair.cdr;
-                Object name = p.car;
-                Declaration decl = null;
-                Pair declForm = null;
-                if (name instanceof String)
-                  {
-                    decl = new Declaration((String) name);
-                    declForm = makePair(p, decl, p.cdr);
-                    st = makePair(st_pair, syntax, declForm);
-                  }
-                else if (name instanceof Pair)
-                  {
-                    Pair name_pair = (Pair) name;
-                    if (name_pair.car instanceof String)
-                      {
-                        decl = new Declaration((String) name_pair.car);
-                        declForm = makePair(name_pair, decl, name_pair.cdr);
-                        p = makePair(p, declForm, p.cdr);
-                        st = makePair(st_pair, syntax, p);
-                      }
-		    }
-                if (decl != null)
-                  {
-                    if (defs instanceof ModuleExp)
-                      {
-                        mustCompileHere();
-                        push(decl);
-                        if (! makePrivate)
-                          {
-                            decl.setCanRead(true);
-                            // decl.setCanWrite(true);
-                          }
-                      }
-                    if (declForm instanceof PairWithPosition)
-                      {
-                        PairWithPosition declPos = (PairWithPosition) declForm;
-                        decl.setFile(declPos.getFile());
-                        decl.setLine(declPos.getLine(), declPos.getColumn());
-                      }
-                    defs.addDeclaration(decl);
-                  }
-                forms.addElement (st);
-              }
-            else if (syntax == Scheme.defineAliasSyntax
-                     && st_pair.cdr instanceof Pair
-                     && ! (current_scope instanceof ModuleExp)
-                     &&  ((Pair) st_pair.cdr).car instanceof String)
-              {
-                Object name = ((Pair) st_pair.cdr).car;
-                Type typeLocation = ClassType.make("gnu.mapping.Location");
-                Declaration decl
-                  = defs.addDeclaration((String) name, typeLocation);
-                decl.setIndirectBinding(true);
-                forms.addElement (st);
-              }
-            else if (syntax == Scheme.defineSyntaxSyntax
-                     && st_pair.cdr instanceof Pair
-                     && ! (current_scope instanceof ModuleExp)
-                     &&  ((Pair) st_pair.cdr).car instanceof String)
-              {
-                Pair p = (Pair) st_pair.cdr;
-                Object name = p.car;
-                if (! (p.car instanceof String)
-                    || ! (p.cdr instanceof Pair)
-                    || (p = (Pair) p.cdr).cdr != List.Empty)
-                  {
-                    forms.addElement(syntaxError("invalid syntax for define-syntax"));
-                    return false;
-                  }
-                Macro macro = new Macro((String) name, p.car);
-                defs.addDeclaration(macro);
-                p = makePair(st_pair, syntax, new Pair(macro, p));
-                forms.addElement (p);
-              }
-            else
-              forms.addElement (st);
-          }
-        return true;
+        Pair st_pair = (Pair) st;
+        Object op = st_pair.car;
+        Syntax syntax = check_if_Syntax (op);
+	if (syntax == null)
+	  forms.addElement(st);
+	else
+	  {
+	    String save_filename = current_filename;
+	    int save_line = current_line;
+	    int save_column = current_column;
+	    try
+	      {
+		if (st_pair instanceof PairWithPosition)
+		  {
+		    PairWithPosition pp = (PairWithPosition) st_pair;
+		    current_filename = pp.getFile();
+		    current_line = pp.getLine();
+		    current_column = pp.getColumn();
+		  }
+		if (! syntax.scanForDefinitions(st_pair, forms, defs, this))
+		  return false;
+	      }
+	    finally
+	      {
+		current_filename = save_filename;
+		current_line = save_line;
+		current_column = save_column;
+	      }
+	  }
       }
+    return true;
   }
 
   /** Recursive helper method for rewrite_body.
@@ -551,7 +426,8 @@ public class Translator extends Object
 
   public boolean scan_body (Object body, java.util.Vector forms, ScopeExp defs)
   {
-    while (body != List.Empty)
+    boolean result = true;
+    while (body != LList.Empty)
       {
 	if (! (body instanceof Pair))
 	  {
@@ -561,10 +437,10 @@ public class Translator extends Object
 	Pair pair = (Pair) body;
 	Object st = pair.car;
         if (! scan_form (st, forms, defs))
-          return false;
+          result = false;
 	body = pair.cdr;
       }
-    return true;
+    return result;
   }
 
   public static Pair makePair(Pair pair, Object car, Object cdr)
@@ -618,8 +494,13 @@ public class Translator extends Object
     return defs;
   }
 
+  ModuleExp module;
+
+  public final ModuleExp getModule() { return module; }
+
   public void finishModule(ModuleExp mexp, java.util.Vector forms)
   {
+    module = mexp;
     int nforms = forms.size();
     int ndecls = mexp.countDecls();
     pushDecls(mexp);
@@ -645,58 +526,74 @@ public class Translator extends Object
     */
   }
 
-  /**
-   * Insert decl into current_decls.
-   * (Used at rewrite time, not eval time.)
-   */
-  void push (Declaration decl)
+  /** Used to remember shadowed bindings in environ.
+   * For each binding, we push <old binding>, <name>.
+   * For each new scope, we push <null>. */
+  java.util.Stack shadowStack = new java.util.Stack();
+
+  /** Note a new binding, remembering old binding in the shadowStack. */
+  public void pushBinding(String name, Object value)
   {
-    String sym = decl.symbol();
-    if (sym == null)
-      return;
-    Object old_decl = current_decls.get (sym);
-    if (old_decl != null)
-      decl.shadowed = old_decl;
-    current_decls.put (sym, decl);
+    Object old = environ.put(name, value);
+    shadowStack.push(old);
+    shadowStack.push(name);
   }
 
-  /** Remove this from Translator.current_decls.
+  public boolean popBinding()
+  {
+    Object name = shadowStack.pop();
+    if (name == null)
+        return false;
+    Object old = shadowStack.pop();
+    if (old == null)
+      environ.remove(name);
+    else
+      environ.put(name, old);
+    return true;
+  }
+
+  /**
+   * Insert decl into environ.
+   * (Used at rewrite time, not eval time.)
+   */
+  public void push (Declaration decl)
+  {
+    String sym = decl.getName();
+    if (sym == null)
+      return;
+    pushBinding(sym, decl);
+  }
+
+  /** Remove this from Translator.environ.
    * (Used at rewrite time, not eval time.)
    */
   void pop (Declaration decl)
   {
-    String sym = decl.symbol();
+    String sym = decl.getName();
     if (sym == null)
       return;
-    if (decl.shadowed == null)
-      current_decls.remove (sym);
-    else
-      current_decls.put (sym, decl.shadowed);
+    popBinding();
   }
 
   public final void pushDecls (ScopeExp scope)
   {
-    for (Variable var = scope.firstVar();  var != null;  var = var.nextVar())
-      {
-	// Don't make artificial variables visible.
-	if (! var.isArtificial())
-	  push((Declaration)var);
-      }
+    //shadowStack.push(null);
+    for (Declaration decl = scope.firstDecl();
+         decl != null;  decl = decl.nextDecl())
+      push(decl);
   }
 
   public final void popDecls (ScopeExp scope)
   {
-    for (Variable var = scope.firstVar();  var != null;  var = var.nextVar())
-      {
-	if (! var.isArtificial ())
-	  pop((Declaration)var);
-      }
+    for (Declaration decl = scope.firstDecl();
+         decl != null;  decl = decl.nextDecl())
+      pop(decl);
   }
 
   public void push (ScopeExp scope)
   {
     scope.outer = current_scope;
-    if (! (scope instanceof LambdaExp)) // which implies: outer != null
+    if (! (scope instanceof ModuleExp))
       mustCompileHere();
     current_scope = scope;
     pushDecls(scope);
@@ -707,5 +604,4 @@ public class Translator extends Object
     popDecls(scope);
     current_scope = scope.outer;
   }
-
 }
