@@ -39,7 +39,7 @@
       (write-char ch out))))
 
 (define (create-directory dirname)
-  ((primitive-virtual-method "java.io.File" "mkdir" "boolean" ())
+  ((primitive-virtual-method <java.io.File> "mkdir" <boolean> ())
    (->pathname dirname)))
 
 ; Taken from MIT Scheme
@@ -66,7 +66,7 @@
 ; From scsh
 ;(define (directory-files [dir [dotfiles?]]) ...)
 
-(define *temp-file-number* 1)
+(define-private *temp-file-number* 1)
 ; From MzLib.  Scsh has (create-temp-file [prefix]).
 ; This is not safe from race conditions!
 ; Fix this using new Java2 File.createTempFile.  FIXME.
