@@ -1,6 +1,9 @@
 (define (pair? x)
   (instance? x <pair>))
 
+(define (cons car cdr)
+  (make <pair> car cdr))
+
 (define (null? x)
   (eq? x '()))
 
@@ -12,8 +15,5 @@
   ((primitive-set-field <pair> "cdr" <object>)
    p x))
 
-#|
-(define (length list)
-  ((primitive-static-method "kawa.lang.List" "length" "int" ())
-   list))
-|#
+(define (length list :: <list>) <int>
+  (invoke-static <list> 'length list))
