@@ -13,8 +13,7 @@
   (instance? x <java.lang.String>))
 
 (define (symbol->string (s <symbol>))
-  ((primitive-constructor <kawa.lang.FString> (<symbol>))
-   s))
+  (make <string> s))
 
 (define (string->symbol (str <string>))
   ((primitive-virtual-method <String> "intern" <symbol> ())
@@ -24,6 +23,9 @@
 (define (procedure? x)
   (instance? x <function>))
 
+(define (values #!rest (args :: <Object[]>))
+  (invoke-static <gnu.mapping.Values> 'make args))
+
 (define (environment-bound? env sym)
   (not (eq? ((primitive-virtual-method "gnu.mapping.Environment" "lookup"
                                        "gnu.mapping.Binding"
@@ -31,6 +33,12 @@
              env sym)
        #!null)))
 
+(define (null-environment)
+  (static-field <kawa.standard.Scheme> 'nullEnvironment))
+
+(define (interaction-environment)
+  (invoke-static <gnu.mapping.Environment> 'user))
+
 (define (scheme-implementation-version)
   (constant-fold
    (primitive-constructor <string> (<java.lang.String>))
@@ -38,7 +46,7 @@
 					   <java.lang.String> ()))))
 #|
 (define (scheme-window #!optional share)
-  ((primitive-constructor <kawa.GuiConsole> (<kawa.lang.Interpreter>))
+  ((primitive-constructor <kawa.GuiConsole> (<gnu.expr.Interpreter>))
    (if share
        ((primitive-constructor <kawa.standard.Scheme>(<gnu.mapping.Environment>))
 	(interaction-environment))
