@@ -10,6 +10,35 @@
 (define (current-output-port)
   ((primitive-static-method <output-port> "outDefault" <output-port> ())))
 
+(define (write value #!optional
+	       (port :: <output-port>
+		     (invoke-static  <output-port> 'outDefault)))
+  (invoke port 'writeSchemeObject value #t))
+
+(define (display value #!optional
+		 (port :: <output-port>
+		       (invoke-static  <output-port> 'outDefault)))
+  (invoke port 'writeSchemeObject value #f))
+
+(define (write-char ch #!optional
+		    (port :: <output-port>
+			  (invoke-static  <output-port> 'outDefault)))
+  (invoke port 'writeSchemeObject ch #f))
+
+;; SRFI-6
+(define (open-input-string string)
+  ((primitive-virtual-method <string> "open" <input-port> ())
+   string))
+
+(define (open-output-string) <string-output-port>
+  ((primitive-constructor  <string-output-port> ())))
+
+(define (get-output-string (output-port  <string-output-port>))
+  ((primitive-constructor <string> (<char[]>))
+   ((primitive-virtual-method <string-output-port> "toCharArray"
+                              <char[]> ())
+    output-port)))
+
 (define (call-with-input-string str proc)
   (let* ((port
 	  ((primitive-virtual-method <string> "open" <input-port> ())
@@ -77,8 +106,11 @@
 			       <void> (<gnu.mapping.Procedure>))
    port prompter))
 
-(define (close-output-port port)
-  ((primitive-virtual-method <output-port> "close" <void> ()) port))
+(define (close-input-port (port :: <input-port>))
+  (invoke port 'close))
+
+(define (close-output-port (port :: <output-port>))
+  (invoke port 'close))
 
 (define (transcript-on filename)
   ((primitive-static-method <output-port> "setLogFile" <void>
