@@ -4,11 +4,16 @@
                  (define-syntax name
                    (lambda (__arg)
                      (syntax-case __arg ()
-                                  ((__name . pattern) form ...)))))))
+                                  ((__name . pattern) (begin form ...))))))))
 
 (define (%defmacro form rule)
   (rule (car (form 'form))))
 
+;; A future version may allow (this [<CLASS>])
+(define-syntax this
+  (lambda (__arg)
+    (make <gnu.expr.ThisExp>)))
+
 (define (gentemp)
   ((primitive-static-method <gnu.expr.Symbol> "gentemp"
 			    <symbol> ())))
