@@ -5,6 +5,7 @@ import java.io.*;
 import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.text.SourceMessages;
+import gnu.kawa.util.*;
 
 /** Start a "Read-Eval-Print-Loop" for the Kawa Scheme evaluator. */
 
@@ -39,7 +40,7 @@ public class repl extends Procedure0or1
     System.exit (-1);
   }
 
-  public static Vector commandLineArguments;
+  public static FVector commandLineArguments;
 
   public static String homeDirectory;
 
@@ -72,37 +73,35 @@ public class repl extends Procedure0or1
     Object[] array = new Object[args.length - arg_start];
     for (int i = arg_start;  i < args.length;  i++)
       array[i - arg_start] = new FString (args[i]);
-    commandLineArguments = new Vector (array);  // FIXME scsh has list
+    commandLineArguments = new FVector (array);  // FIXME scsh has list
     // FIXME scsh also has command-line proc
     Environment.define_global ("command-line-arguments",
 			       commandLineArguments);
   }
 
-  static Interpreter defaultInterpreter = null;
-
   public static Interpreter getInterpreter()
   {
-    if (defaultInterpreter == null)
+    if (Interpreter.defaultInterpreter == null)
       {
-	defaultInterpreter = new Scheme();
-	Environment.setCurrent(defaultInterpreter.getEnvironment());
+	Interpreter.defaultInterpreter = new Scheme();
+	Environment.setCurrent(Interpreter.defaultInterpreter.getEnvironment());
       }
-    return defaultInterpreter;
+    return Interpreter.defaultInterpreter;
   }
 
   public static void setInterpreter(String name)
   {
-    Interpreter previous = defaultInterpreter; 
+    Interpreter previous = Interpreter.defaultInterpreter; 
     if (name.length() > 2 && name.charAt(0) == '-')
       name = name.substring(name.charAt(1) == '-' ? 2 :1);
     if (name.equals("scheme"))
-      defaultInterpreter = Scheme.getInstance();
+      Interpreter.defaultInterpreter = Scheme.getInstance();
     else if (name.equals("elisp") || name.equals("emacs"))
-      defaultInterpreter = gnu.jemacs.lang.ELisp.getInstance();
+      Interpreter.defaultInterpreter = gnu.jemacs.lang.ELisp.getInstance();
     else
       bad_option(name);
     if (previous == null)
-      Environment.setCurrent(defaultInterpreter.getEnvironment());
+      Environment.setCurrent(Interpreter.defaultInterpreter.getEnvironment());
   }
 
   public static void main(String args[])
@@ -121,7 +120,7 @@ public class repl extends Procedure0or1
 	    setArgs (args, iArg+1);
 	    if (arg.equals ("-c"))
 	      checkInitFile();
-	    Shell.runString(args[iArg], defaultInterpreter, Environment.getCurrent());
+	    Shell.runString(args[iArg], Interpreter.defaultInterpreter, Environment.getCurrent());
 	    something_done = true;
 	  }
 	else if (arg.equals ("-f"))
@@ -141,7 +140,7 @@ public class repl extends Procedure0or1
 	    getInterpreter();
 	    setArgs (args, iArg);
 	    checkInitFile();
-	    Shell.run(defaultInterpreter, Environment.getCurrent());
+	    Shell.run(Interpreter.defaultInterpreter, Environment.getCurrent());
 	    return;
 	  }
 	else if (arg.equals ("-w"))
@@ -149,7 +148,17 @@ public class repl extends Procedure0or1
 	    getInterpreter();
 	    setArgs (args, iArg);
 	    checkInitFile();
-	    new kawa.GuiConsole(defaultInterpreter, Environment.getCurrent());
+	    // Do this instead of just new GuiConsole in case we have
+	    // configured --without-awt.
+	    try
+	      {
+		Class.forName("kawa.GuiConsole").newInstance();
+	      }
+	    catch (Exception ex)
+	      {
+		System.err.println("failed to create Kawa window: "+ex);
+		System.exit (-1);
+	      }
 	    something_done = true;
 	  }
 	else if (arg.equals ("-d"))
@@ -278,7 +287,7 @@ public class repl extends Procedure0or1
 		    System.err.println("got connection from "
 				       +client.getInetAddress()
 				       +" port:"+client.getPort());
-		    serveTelnet(defaultInterpreter, client);
+		    serveTelnet(Interpreter.defaultInterpreter, client);
 		  }
 	      }
 	    catch (java.io.IOException ex)
@@ -292,6 +301,10 @@ public class repl extends Procedure0or1
 	  {
 	    Compilation.generateMainDefault = true;
 	  }
+	else if (arg.equals("--applet"))
+	  {
+	    Compilation.generateAppletDefault = true;
+	  }
 	else if (arg.equals("--debug-dump-zip"))
 	  {
 	    gnu.expr.LambdaExp.dumpZipPrefix = "kawa-zip-dump-";
@@ -318,6 +331,7 @@ public class repl extends Procedure0or1
 	    System.out.print("Kawa ");
 	    System.out.print(Version.getVersion());
 	    System.out.println();
+            System.out.println("Copyright (C) 1999 Per Bothner");
 	    something_done = true;
 	  }
 	else if (arg.length () > 0 && arg.charAt(0) == '-')
@@ -338,7 +352,7 @@ public class repl extends Procedure0or1
       {
 	setArgs (args, iArg);
 	checkInitFile();
-	Shell.run(defaultInterpreter);
+	Shell.run(Interpreter.defaultInterpreter);
       }
    }
 
