@@ -7,15 +7,17 @@ import gnu.mapping.*;
 import gnu.expr.*;
 import java.util.Hashtable;
 import gnu.text.SourceMessages;
+import gnu.text.CharArrayInPort;
+import gnu.kawa.util.*;
 
 public class Scheme extends Interpreter
 {
-  final void define_proc (Named proc)
+  protected void define_proc (Named proc)
   {
-    define (proc.name (), proc);
+    define (proc.getName (), proc);
   }
 
-  final void define_proc (String name, Named proc)
+  protected void define_proc (String name, Named proc)
   {
     if (proc.getName() == null)
       proc.setName(name);
@@ -23,12 +25,12 @@ public class Scheme extends Interpreter
   }
 
   /* Define a procedure to be autoloaded. */
-  final void define_proc (String name, String className)
+  protected void define_proc (String name, String className)
   {
     define (name, new AutoloadProcedure (name, className));
   }
 
-  final void define_syntax (String name, Syntax proc)
+  protected void define_syntax (String name, Syntax proc)
   {
     if (proc.getName() == null)
       proc.setName(name);
@@ -36,21 +38,15 @@ public class Scheme extends Interpreter
   }
 
   /* Define a Syntax to be autoloaded. */
-  final void define_syntax (String name, String className)
+  protected void define_syntax (String name, String className)
   {
     define (name, new AutoloadSyntax (name, className));
   }
 
-  static Environment null_environment;
-  static Environment r4_environment;
-  static Environment r5_environment;
-  protected static Environment kawa_environment;
-
-  public static Syntax beginSyntax;
-  public static Syntax defineSyntax;
-  public static Syntax defineSyntaxPrivate;
-  public static Syntax defineAliasSyntax;
-  public static Syntax defineSyntaxSyntax;
+  public static Environment nullEnvironment;
+  static Environment r4Environment;
+  static Environment r5Environment;
+  protected static Environment kawaEnvironment;
 
   public static SpecialType byteType
     = new SpecialType ("byte", "B", 1, java.lang.Byte.TYPE);
@@ -76,16 +72,16 @@ public class Scheme extends Interpreter
 
   public static Scheme getInstance()
   {
-    if (kawa_environment == null)
+    if (kawaEnvironment == null)
       new Scheme ();
     return instance;
   }
 
   public static synchronized Environment builtin ()
   {
-    if (kawa_environment == null)
+    if (kawaEnvironment == null)
       new Scheme ();
-    return kawa_environment;
+    return kawaEnvironment;
   }
 
   public void initScheme ()
@@ -98,15 +94,14 @@ public class Scheme extends Interpreter
       Procedure2 equal;
 
       // (null-environment)
-      null_environment = new Environment ();
-      null_environment.setName ("null-environment");
-      environ = null_environment;
+      nullEnvironment = new Environment ();
+      nullEnvironment.setName ("null-environment");
+      environ = nullEnvironment;
 
       //-- Section 4.1  -- complete
       define (Interpreter.quote_sym, new Quote ());
-      define_syntax("define", defineSyntax = new kawa.standard.define(false));
-      define_syntax("define-private",
-                    defineSyntaxPrivate = new kawa.standard.define(true));
+      define_syntax("define", new kawa.standard.define(false));
+      define_syntax("define-private", new kawa.standard.define(true));
       define_syntax ("if", "kawa.standard.ifp");
       define_syntax ("set!", "kawa.standard.set_b");
 
@@ -114,7 +109,7 @@ public class Scheme extends Interpreter
       define_syntax ("cond", "kawa.lib.std_syntax");
       define_syntax ("case", "kawa.lib.std_syntax");
       define_syntax ("and", "kawa.lib.std_syntax");
-      define ("or", new kawa.standard.and_or (false));
+      define ("or", new kawa.standard.and_or (false, this));
       define_syntax ("%let", "kawa.standard.let");
       define_syntax ("let", "kawa.lib.std_syntax");
       define_syntax ("%let-decl", "kawa.lib.std_syntax");
@@ -122,7 +117,7 @@ public class Scheme extends Interpreter
       define_syntax ("let*", "kawa.lib.std_syntax");
       define_syntax ("letrec", "kawa.standard.letrec");
 
-      define ("begin", beginSyntax = new kawa.standard.begin());
+      define ("begin", new kawa.standard.begin());
       define_syntax ("do", "kawa.lib.std_syntax");
       define_syntax ("delay", "kawa.lib.std_syntax");
       define_proc ("%make-promise", "kawa.lib.std_syntax");
@@ -132,19 +127,18 @@ public class Scheme extends Interpreter
       define_syntax ("lambda", "kawa.lang.Lambda");
 
       // Appendix (and R5RS)
-      defineSyntaxSyntax = new kawa.standard.define_syntax ();
-      define ("define-syntax", defineSyntaxSyntax);
+      define ("define-syntax", new kawa.standard.define_syntax ());
       define ("syntax-rules", new kawa.standard.syntax_rules ());
       define ("syntax-case", new kawa.standard.syntax_case ());
       define ("let-syntax", new kawa.standard.let_syntax (false));
       define ("letrec-syntax", new kawa.standard.let_syntax (true));
 
-      r4_environment = new Environment (null_environment);
-      r4_environment.setName ("r4rs-environment");
-      environ = r4_environment;
+      r4Environment = new Environment (nullEnvironment);
+      r4Environment.setName ("r4rs-environment");
+      environ = r4Environment;
 
       //-- Section 6.1  -- complete
-      define_proc ("not", new kawa.standard.not());
+      define_proc ("not", new kawa.standard.not(this));
       define_proc ("boolean?", "kawa.lib.misc");
 
       //-- Section 6.2  -- complete
@@ -157,7 +151,7 @@ public class Scheme extends Interpreter
 
       //-- Section 6.3  -- complete
       define_proc("pair?", "kawa.lib.lists");
-      define_proc("cons", kawa.standard.cons.consProcedure);
+      define_proc("cons", "kawa.lib.lists");
       define_proc ("car", "kawa.standard.car");
       define_proc ("cdr", "kawa.standard.cdr");
       define_proc ("set-car!", "kawa.lib.lists");
@@ -194,24 +188,24 @@ public class Scheme extends Interpreter
       define_proc ("null?", "kawa.lib.lists");
       define_proc ("list?", "kawa.standard.list_p");
       define_proc ("list", "kawa.standard.list_v");
-      define_proc ("length", "kawa.standard.length");
-      define ("append", kawa.standard.append.appendProcedure);
+      define_proc ("length", "kawa.lib.lists");
+      define_proc ("append", "kawa.standard.append");
       define_proc ("reverse", "kawa.standard.reverse");
       define_proc ("list-tail", "kawa.standard.list_tail");
       define_proc ("list-ref", "kawa.standard.list_ref");
 
       proc = new kawa.standard.mem("memq",eq);
-      define(proc.name (), proc);
+      define(proc.getName (), proc);
       proc = new kawa.standard.mem("memv",eqv);
-      define(proc.name (), proc);
+      define(proc.getName (), proc);
       proc = new kawa.standard.mem("member",equal);
-      define(proc.name (), proc);
+      define(proc.getName (), proc);
       proc = new kawa.standard.ass("assq",eq);
-      define(proc.name (), proc);
+      define(proc.getName (), proc);
       proc = new kawa.standard.ass("assv",eqv);
-      define(proc.name (), proc);
+      define(proc.getName (), proc);
       proc = new kawa.standard.ass("assoc",equal);
-      define(proc.name (), proc);
+      define(proc.getName (), proc);
 
       //-- Section 6.4  -- complete, including slashified read/write
       
@@ -221,18 +215,18 @@ public class Scheme extends Interpreter
 
       //-- Section 6.5
       define_proc ("number?", "kawa.lib.numbers");
-      define_proc ("quantity?", "kawa.lib.number");
+      define_proc ("quantity?", "kawa.lib.numbers");
       define_proc ("complex?", "kawa.lib.numbers");
       define_proc ("real?", "kawa.lib.numbers");
       define_proc ("rational?", "kawa.lib.numbers");
       define_proc ("integer?", "kawa.standard.integer_p");
       define_proc ("exact?", "kawa.standard.exact_p");
       define_proc ("inexact?", "kawa.standard.inexact_p");
-      define_proc ("=", "kawa.standard.equal_oper");
-      define_proc ("<", "kawa.standard.less_oper");
-      define_proc (">", "kawa.standard.greater_oper");
-      define_proc ("<=", "kawa.standard.lessequal_oper");
-      define_proc (">=", "kawa.standard.greaterequal_oper");
+      define_proc ("=", NumberCompare.$Eq);
+      define_proc ("<", NumberCompare.$Ls);
+      define_proc (">", NumberCompare.$Gr);
+      define_proc ("<=", NumberCompare.$Ls$Eq);
+      define_proc (">=", NumberCompare.$Gr$Eq);
       define_proc ("zero?", "kawa.lib.numbers");
       define_proc ("positive?", "kawa.standard.positive_p");
       define_proc ("negative?", "kawa.lib.numbers");
@@ -262,8 +256,8 @@ public class Scheme extends Interpreter
       define_proc ("sin", "kawa.lib.numbers");
       define_proc ("cos", "kawa.lib.numbers");
       define_proc ("tan", "kawa.lib.numbers");
-      define_proc ("asin", "kawa.standard.asin");
-      define_proc ("acos", "kawa.standard.acos");
+      define_proc ("asin", "kawa.lib.numbers");
+      define_proc ("acos", "kawa.lib.numbers");
       define_proc ("atan", "kawa.standard.atan");
       define_proc ("sqrt", "kawa.standard.sqrt");
       define_proc ("expt", "kawa.standard.expt");
@@ -329,16 +323,16 @@ public class Scheme extends Interpreter
 
       //-- Section 6.8  -- complete
       define_proc ("vector?", "kawa.lib.vectors");
-      define_proc ("make-vector", "kawa.standard.make_vector");
-      define ("vector", kawa.standard.vector_v.vectorProcedure);
+      define_proc ("make-vector", "kawa.lib.vectors");
+      define_proc ("vector", "kawa.lib.vectors");
       define_proc ("vector-length", "kawa.lib.vectors");
       define_proc ("vector-ref", "kawa.lib.vectors");
       define_proc ("vector-set!", "kawa.lib.vectors");
       define_proc ("list->vector", "kawa.lib.vectors");
       define_proc ("vector->list", "kawa.standard.vector2list");
-      define_proc ("vector-fill!", "kawa.standard.vector_fill_b");
+      define_proc ("vector-fill!", "kawa.lib.vectors");
       // Extension:
-      define ("vector-append", kawa.standard.vector_append.vappendProcedure);
+      define_proc ("vector-append", "kawa.standard.vector_append");
 
       //-- Section 6.9  -- complete [except restricted call/cc]
       define_proc ("procedure?", "kawa.lib.misc");
@@ -364,7 +358,7 @@ public class Scheme extends Interpreter
 		   "kawa.standard.with_output_to_file");
       define_proc ("open-input-file", "kawa.standard.open_input_file");
       define_proc ("open-output-file", "kawa.standard.open_output_file");
-      define_proc ("close-input-port", "kawa.standard.close_input_port");
+      define_proc ("close-input-port", "kawa.lib.ports");
       define_proc ("close-output-port", "kawa.lib.ports");
       define_proc ("read", "kawa.standard.read");
       define_proc ("read-line", "kawa.standard.read_line");
@@ -372,14 +366,17 @@ public class Scheme extends Interpreter
       define_proc (new readchar (true));   // peek-char
       define_proc ("eof-object?", "kawa.lib.ports");
       define_proc ("char-ready?", "kawa.standard.char_ready_p");
-      define_proc (new write(true));       // write
-      define_proc (new write(false));      // display
-      define_proc ("write-char", "kawa.standard.writechar");
+      define_proc ("write", "kawa.lib.ports");
+      define_proc ("display", "kawa.lib.ports");
+      define_proc ("write-char", "kawa.lib.ports");
       define_proc ("newline", "kawa.lib.ports");
       define_proc ("load", "kawa.standard.load");
       define_proc ("transcript-off", "kawa.lib.ports");
       define_proc ("transcript-on", "kawa.lib.ports");
       define_proc ("call-with-input-string", "kawa.lib.ports");  // Extension
+      define_proc ("open-input-string", "kawa.lib.ports");  // SRFI-6
+      define_proc ("open-output-string", "kawa.lib.ports");  // SRFI-6
+      define_proc ("get-output-string", "kawa.lib.ports");  // SRFI-6
       define_proc ("call-with-output-string",  // Extension
 		   "kawa.standard.call_with_output_string");
       define_proc ("force-output", "kawa.lib.ports");  // Extension
@@ -397,20 +394,20 @@ public class Scheme extends Interpreter
 
       define_syntax ("%syntax-error", "kawa.standard.syntax_error");
 
-      r5_environment = new Environment (r4_environment);
-      r5_environment.setName ("r5rs-environment");
-      environ = r5_environment;
-      define_proc ("values", "kawa.standard.values_v");
+      r5Environment = new Environment (r4Environment);
+      r5Environment.setName ("r5rs-environment");
+      environ = r5Environment;
+      define_proc ("values", "kawa.lib.misc");
       define_proc ("call-with-values", "kawa.standard.call_with_values");
       define_proc ("eval", "kawa.lang.Eval");
       define_proc ("repl", new kawa.repl(this));
       define_proc ("scheme-report-environment", "kawa.standard.scheme_env");
-      define_proc ("null-environment", "kawa.standard.null_env");
-      define_proc ("interaction-environment", "kawa.standard.user_env");
+      define_proc ("null-environment", "kawa.lib.misc");
+      define_proc ("interaction-environment", "kawa.lib.misc");
       define_proc ("dynamic-wind", "kawa.lib.syntax");
 
-      kawa_environment = new Environment (r5_environment);
-      environ = kawa_environment;
+      kawaEnvironment = new Environment (r5Environment);
+      environ = kawaEnvironment;
 
       define_proc ("exit", "kawa.lib.thread");
 
@@ -421,13 +418,13 @@ public class Scheme extends Interpreter
       define_proc ("logand", "kawa.standard.logand");
       define_proc ("logior", "kawa.standard.logior");
       define_proc ("logxor", "kawa.standard.logxor");
-      define_proc ("lognot", "kawa.standard.lognot");
-      define_proc ("logop", "kawa.standard.logop");
-      define_proc ("logbit?", "kawa.standard.logbit_p");
-      define_proc ("logtest", "kawa.standard.logtest");
-      define_proc ("logcount", "kawa.standard.logcount");
-      define_proc ("bit-extract", "kawa.standard.bit_extract");
-      define_proc ("integer-length", "kawa.standard.int_length");
+      define_proc ("lognot", "kawa.lib.numbers");
+      define_proc ("logop", "kawa.lib.numbers");
+      define_proc ("logbit?", "kawa.lib.numbers");
+      define_proc ("logtest", "kawa.lib.numbers");
+      define_proc ("logcount", "kawa.lib.numbers");
+      define_proc ("bit-extract", "kawa.lib.numbers");
+      define_proc ("integer-length", "kawa.lib.numbers");
 
       // These are from SLIB.
       define_proc("string-upcase!", "kawa.lib.strings");
@@ -436,7 +433,6 @@ public class Scheme extends Interpreter
       define_proc("string-upcase", "kawa.lib.strings");
       define_proc("string-downcase", "kawa.lib.strings");
       define_proc("string-capitalize", "kawa.lib.strings");
-
       define_syntax("primitive-virtual-method",
                     new kawa.standard.prim_method(182));
       define_syntax("primitive-static-method",
@@ -465,13 +461,16 @@ public class Scheme extends Interpreter
       define_proc("instance?", new kawa.standard.instance());
       define_syntax("synchronized", "kawa.standard.synchronizd");
       define_syntax("object", "kawa.standard.object");
-      define_proc("make", "gnu.kawa.reflect.MakeInstance");
-      define_proc("slot-ref", "gnu.kawa.reflect.SlotGet");
+      define_syntax("define-class", "kawa.standard.define_class");
+      define_syntax("this", "kawa.lib.syntax");
+      define_proc("make", gnu.kawa.reflect.Invoke.make);
+      define_proc("slot-ref", gnu.kawa.reflect.SlotGet.field);
       define_proc("slot-set!", "gnu.kawa.reflect.SlotSet");
-      define_proc("field", "gnu.kawa.reflect.SlotGet");
+      define_proc("field", gnu.kawa.reflect.SlotGet.field);
       define_proc("class-methods", "gnu.kawa.reflect.ClassMethods");
-      define_proc("static-field", "kawa.standard.static_field");
-      define_proc("invoke-static", "gnu.kawa.reflect.InvokeStatic");
+      define_proc("static-field", gnu.kawa.reflect.SlotGet.staticField);
+      define_proc("invoke", gnu.kawa.reflect.Invoke.invoke);
+      define_proc("invoke-static", gnu.kawa.reflect.Invoke.invokeStatic);
 
       define_proc("file-exists?", "kawa.lib.files");
       define_proc("file-directory?", "kawa.lib.files");
@@ -542,19 +541,110 @@ public class Scheme extends Interpreter
       define_proc ("keyword?", "kawa.lib.keywords");
       define_proc ("keyword->string", "kawa.lib.keywords");
       define_proc ("string->keyword", "kawa.lib.keywords");
-      define_proc ("%makeProcLocation", "kawa.standard.makeProcLocation");
       define_syntax ("location", "kawa.standard.location");
-      defineAliasSyntax = new kawa.standard.define_alias();
-      define ("define-alias", defineAliasSyntax);
+      define ("define-alias", new kawa.standard.define_alias());
+      define ("define-variable", new kawa.standard.define_variable());
+      define ("define-member-alias", new kawa.standard.define_member_alias());
+      define ("require", new kawa.standard.require());
+      define_syntax ("module-name", "kawa.standard.module_name");
+      define_syntax ("module-extends", "kawa.standard.module_extends");
+      define_syntax ("module-implements", "kawa.standard.module_implements");
+
+      define_proc ("s8vector?", "kawa.lib.uniform");
+      define_proc ("make-s8vector", "kawa.lib.uniform");
+      define_proc ("s8vector", "kawa.lib.uniform");
+      define_proc ("s8vector-length", "kawa.lib.uniform");
+      define_proc ("s8vector-ref", "kawa.lib.uniform");
+      define_proc ("s8vector-set!", "kawa.lib.uniform");
+      define_proc ("s8vector->list", "kawa.lib.uniform");
+      define_proc ("list->s8vector", "kawa.lib.uniform");
+      define_proc ("u8vector?", "kawa.lib.uniform");
+      define_proc ("make-u8vector", "kawa.lib.uniform");
+      define_proc ("u8vector", "kawa.lib.uniform");
+      define_proc ("u8vector-length", "kawa.lib.uniform");
+      define_proc ("u8vector-ref", "kawa.lib.uniform");
+      define_proc ("u8vector-set!", "kawa.lib.uniform");
+      define_proc ("u8vector->list", "kawa.lib.uniform");
+      define_proc ("list->u8vector", "kawa.lib.uniform");
+
+      define_proc ("s16vector?", "kawa.lib.uniform");
+      define_proc ("make-s16vector", "kawa.lib.uniform");
+      define_proc ("s16vector", "kawa.lib.uniform");
+      define_proc ("s16vector-length", "kawa.lib.uniform");
+      define_proc ("s16vector-ref", "kawa.lib.uniform");
+      define_proc ("s16vector-set!", "kawa.lib.uniform");
+      define_proc ("s16vector->list", "kawa.lib.uniform");
+      define_proc ("list->s16vector", "kawa.lib.uniform");
+      define_proc ("u16vector?", "kawa.lib.uniform");
+      define_proc ("make-u16vector", "kawa.lib.uniform");
+      define_proc ("u16vector", "kawa.lib.uniform");
+      define_proc ("u16vector-length", "kawa.lib.uniform");
+      define_proc ("u16vector-ref", "kawa.lib.uniform");
+      define_proc ("u16vector-set!", "kawa.lib.uniform");
+      define_proc ("u16vector->list", "kawa.lib.uniform");
+      define_proc ("list->u16vector", "kawa.lib.uniform");
+
+      define_proc ("s32vector?", "kawa.lib.uniform");
+      define_proc ("make-s32vector", "kawa.lib.uniform");
+      define_proc ("s32vector", "kawa.lib.uniform");
+      define_proc ("s32vector-length", "kawa.lib.uniform");
+      define_proc ("s32vector-ref", "kawa.lib.uniform");
+      define_proc ("s32vector-set!", "kawa.lib.uniform");
+      define_proc ("s32vector->list", "kawa.lib.uniform");
+      define_proc ("list->s32vector", "kawa.lib.uniform");
+      define_proc ("u32vector?", "kawa.lib.uniform");
+      define_proc ("make-u32vector", "kawa.lib.uniform");
+      define_proc ("u32vector", "kawa.lib.uniform");
+      define_proc ("u32vector-length", "kawa.lib.uniform");
+      define_proc ("u32vector-ref", "kawa.lib.uniform");
+      define_proc ("u32vector-set!", "kawa.lib.uniform");
+      define_proc ("u32vector->list", "kawa.lib.uniform");
+      define_proc ("list->u32vector", "kawa.lib.uniform");
+
+      define_proc ("s64vector?", "kawa.lib.uniform");
+      define_proc ("make-s64vector", "kawa.lib.uniform");
+      define_proc ("s64vector", "kawa.lib.uniform");
+      define_proc ("s64vector-length", "kawa.lib.uniform");
+      define_proc ("s64vector-ref", "kawa.lib.uniform");
+      define_proc ("s64vector-set!", "kawa.lib.uniform");
+      define_proc ("s64vector->list", "kawa.lib.uniform");
+      define_proc ("list->s64vector", "kawa.lib.uniform");
+      define_proc ("u64vector?", "kawa.lib.uniform");
+      define_proc ("make-u64vector", "kawa.lib.uniform");
+      define_proc ("u64vector", "kawa.lib.uniform");
+      define_proc ("u64vector-length", "kawa.lib.uniform");
+      define_proc ("u64vector-ref", "kawa.lib.uniform");
+      define_proc ("u64vector-set!", "kawa.lib.uniform");
+      define_proc ("u64vector->list", "kawa.lib.uniform");
+      define_proc ("list->u64vector", "kawa.lib.uniform");
+
+      define_proc ("f32vector?", "kawa.lib.uniform");
+      define_proc ("make-f32vector", "kawa.lib.uniform");
+      define_proc ("f32vector", "kawa.lib.uniform");
+      define_proc ("f32vector-length", "kawa.lib.uniform");
+      define_proc ("f32vector-ref", "kawa.lib.uniform");
+      define_proc ("f32vector-set!", "kawa.lib.uniform");
+      define_proc ("f32vector->list", "kawa.lib.uniform");
+      define_proc ("list->f32vector", "kawa.lib.uniform");
+      define_proc ("f64vector?", "kawa.lib.uniform");
+      define_proc ("make-f64vector", "kawa.lib.uniform");
+      define_proc ("f64vector", "kawa.lib.uniform");
+      define_proc ("f64vector-length", "kawa.lib.uniform");
+      define_proc ("f64vector-ref", "kawa.lib.uniform");
+      define_proc ("f64vector-set!", "kawa.lib.uniform");
+      define_proc ("f64vector->list", "kawa.lib.uniform");
+      define_proc ("list->f64vector", "kawa.lib.uniform");
+
+      define_proc ("emacs", "gnu.jemacs.buffer.emacs");
   }
 
   static int scheme_counter = 0;
 
   public Scheme ()
   {
-    if (kawa_environment == null)
+    if (kawaEnvironment == null)
       initScheme();
-    environ = new ScmEnv (kawa_environment);
+    environ = new ScmEnv (kawaEnvironment);
     environ.setName ("interaction-environment."+(++scheme_counter));
     if (instance == null)
       instance = this;
@@ -565,11 +655,16 @@ public class Scheme extends Interpreter
     this.environ = environ;
   }
 
+  public String getName()
+  {
+    return "Scheme";
+  }
+
   public Environment getNewEnvironment ()
   {
-    if (kawa_environment == null)
+    if (kawaEnvironment == null)
       initScheme();
-    Environment environ = new ScmEnv (kawa_environment);
+    Environment environ = new ScmEnv (kawaEnvironment);
     environ.setName ("interaction-environment."+(++scheme_counter));
     return environ;
   }
@@ -668,7 +763,8 @@ public class Scheme extends Interpreter
     else if (exp instanceof ReferenceExp)
       {
 	ReferenceExp rexp = (ReferenceExp) exp;
-	if (rexp.getBinding() == null)
+        Declaration binding = rexp.getBinding();
+	if (binding == null)
 	  {
 	    String name = rexp.getName();
 	    int len = name.length(); 
@@ -720,17 +816,35 @@ public class Scheme extends Interpreter
 	types.put ("integer", ClassType.make("gnu.math.IntNum"));
 	types.put ("symbol", ClassType.make("java.lang.String"));
 	types.put ("keyword", ClassType.make("gnu.expr.Keyword"));
-	types.put ("list", ClassType.make("kawa.lang.List"));
-	types.put ("pair", ClassType.make("kawa.lang.Pair"));
-	types.put ("string", ClassType.make("kawa.lang.FString"));
-	types.put ("character", ClassType.make("kawa.lang.Char"));
-	types.put ("vector", ClassType.make("kawa.lang.Vector"));
+	types.put ("list", ClassType.make("gnu.kawa.util.LList"));
+	types.put ("pair", ClassType.make("gnu.kawa.util.Pair"));
+	types.put ("string", ClassType.make("gnu.kawa.util.FString"));
+	types.put ("character", ClassType.make("gnu.kawa.util.Char"));
+	types.put ("vector", ClassType.make("gnu.kawa.util.FVector"));
 	types.put ("function", ClassType.make("gnu.mapping.Procedure"));
 	types.put ("input-port", ClassType.make("gnu.mapping.InPort"));
 	types.put ("output-port", ClassType.make("gnu.mapping.OutPort"));
+	types.put ("string-output-port",
+                   ClassType.make("gnu.mapping.CharArrayOutPort"));
 	types.put ("record", ClassType.make("kawa.lang.Record"));
 	types.put ("type", ClassType.make("gnu.bytecode.Type"));
 	types.put ("class-type", ClassType.make("gnu.bytecode.ClassType"));
+
+        types.put ("s8vector", ClassType.make("gnu.kawa.util.S8Vector"));
+        types.put ("u8vector", ClassType.make("gnu.kawa.util.U8Vector"));
+        types.put ("s16vector", ClassType.make("gnu.kawa.util.S16Vector"));
+        types.put ("u16vector", ClassType.make("gnu.kawa.util.U16Vector"));
+        types.put ("s32vector", ClassType.make("gnu.kawa.util.S32Vector"));
+        types.put ("u32vector", ClassType.make("gnu.kawa.util.U32Vector"));
+        types.put ("s64vector", ClassType.make("gnu.kawa.util.S64Vector"));
+        types.put ("u64vector", ClassType.make("gnu.kawa.util.U64Vector"));
+        types.put ("f32vector", ClassType.make("gnu.kawa.util.F32Vector"));
+        types.put ("f64vector", ClassType.make("gnu.kawa.util.F64Vector"));
+
+        // Only if JEmacs is available.  FIXME.
+	types.put ("buffer", ClassType.make("gnu.jemacs.buffer.Buffer"));
+	types.put ("frame", ClassType.make("gnu.jemacs.buffer.Frame"));
+	types.put ("window", ClassType.make("gnu.jemacs.buffer.Window"));
       }
     return (Type) types.get(name);
   }
@@ -755,7 +869,7 @@ public class Scheme extends Interpreter
 	t = new ArrayType(t);
       }
     else if (gnu.bytecode.Type.isValidJavaTypeName(name))
-      t = new ClassType (name);
+      t = ClassType.make(name);
     else
       return null;
     types.put (name, t);
@@ -772,7 +886,7 @@ public class Scheme extends Interpreter
         Object cname = ((QuoteExp) exp).getValue();
         if (cname instanceof Type)
           return (Type) cname;
-        if (cname instanceof kawa.lang.FString || cname instanceof String)
+        if (cname instanceof FString || cname instanceof String)
           return ClassType.make(cname.toString());
       }
     else if (exp instanceof ReferenceExp)
@@ -790,4 +904,12 @@ public class Scheme extends Interpreter
     return null;
   }
 
+  /** The compiler insert calls to this method for applications and applets. */
+  public static void registerEnvironment()
+  {
+    Scheme interp = new Scheme();
+    Interpreter.defaultInterpreter = interp;
+    Environment.setCurrent(interp.getEnvironment());
+  }
+
 }
