@@ -1,11 +1,10 @@
 package kawa.standard;
-import kawa.lang.*;
 import gnu.math.IntNum;
+import gnu.mapping.*;
 
 public class ashift extends Procedure2
 {
   public Object apply2 (Object arg1, Object arg2)
-       throws WrongArguments, WrongType, GenericError
   {
     return IntNum.shift ((IntNum) arg1, IntNum.intValue (arg2));
   }
