@@ -1,7 +1,7 @@
 package kawa.standard;
-import kawa.lang.*;
 import gnu.mapping.Procedure2;
 import gnu.mapping.WrongType;
+import gnu.kawa.util.*;
 
 public class ass extends Procedure2
 {
@@ -33,8 +33,8 @@ public class ass extends Procedure2
 	arg2 = list.cdr;
       }
 
-    if (arg2 == List.Empty)
-      return Interpreter.falseObject;
+    if (arg2 == LList.Empty)
+      return Boolean.FALSE;
     else
       throw new WrongType(this.name (), 2, "list");
   }
