@@ -2,6 +2,7 @@ package kawa.standard;
 import kawa.lang.*;
 import gnu.mapping.*;
 import gnu.expr.*;
+import gnu.kawa.util.*;
 
 /**
  * Implement the re-writer for the "begin" primitive.
@@ -14,4 +15,10 @@ public class begin extends Syntax implements Printable
   {
     return tr.rewrite_body (obj);
   }
+
+  public boolean scanForDefinitions (Pair st, java.util.Vector forms,
+                                    ScopeExp defs, Translator tr)
+  {
+    return tr.scan_body(st.cdr, forms, defs);
+  }
 }
