@@ -1,8 +1,8 @@
 package kawa.standard;
-import kawa.lang.*;
 import gnu.mapping.Procedure1;
 import gnu.mapping.HasSetter;
 import gnu.mapping.WrongType;
+import gnu.kawa.util.Pair;
 
 public class cdr extends Procedure1 implements HasSetter
 {
