@@ -1,9 +1,14 @@
 package kawa.standard;
-import kawa.lang.*;
+import gnu.kawa.util.Char;
 import gnu.mapping.*;
 
 public class char_equal_p extends Procedure2
 {
+  public static boolean apply(Char arg1, Char arg2)
+  {
+    return arg1.intValue () == arg2.intValue();
+  }
+
   public Object apply2(Object arg1, Object arg2)
   {
     if (! (arg1 instanceof Char))
