@@ -3,9 +3,20 @@ import kawa.lang.*;
 import gnu.bytecode.Type;
 import gnu.bytecode.ClassType;
 import gnu.bytecode.CodeAttr;
+import gnu.mapping.*;
+import gnu.expr.*;
 
 public class convert extends Procedure2 implements Inlineable
 {
+  static convert instance = null;
+
+  public static convert getInstance ()
+  {
+    if (instance == null)
+      instance = new convert();
+    return instance;
+  }
+
   public Object apply2 (Object arg1, Object arg2)
   {
     Type type = (Type) arg1;
@@ -15,7 +26,7 @@ public class convert extends Procedure2 implements Inlineable
   static gnu.bytecode.ClassType typeType;
   static gnu.bytecode.Method coerceMethod;
 
-  public void compile (ApplyExp exp, Compilation comp, int flags)
+  public void compile (ApplyExp exp, Compilation comp, Target target)
   {
     Expression[] args = exp.getArgs();
     if (args.length != 2)
@@ -24,9 +35,8 @@ public class convert extends Procedure2 implements Inlineable
     Type type = Scheme.getTypeValue(args[0]);
     if (type != null)
       {
-	args[1].compile(comp, 0);
-	type.emitCoerceFromObject(comp.getCode());
-	
+        args[1].compile(comp, Target.pushValue(type));
+	target.compileFromStack(comp, type);
       }
     else
       {
@@ -39,11 +49,21 @@ public class convert extends Procedure2 implements Inlineable
 					      gnu.bytecode.Access.PUBLIC);
 	  }
 
-	args[0].compile(comp, 0, typeType);
-	args[1].compile(comp, 0);
+	args[0].compile(comp, typeType);
+	args[1].compile(comp, Target.pushObject);
 	code.emitInvokeVirtual(coerceMethod);
+	target.compileFromStack(comp, Type.pointer_type);
       }
-    if ((flags & Expression.IGNORED) != 0)
-      code.emitPop(1);
+  }
+
+  public Type getReturnType (Expression[] args)
+  {
+    if (args != null && args.length == 2)
+      {
+	Type type = Scheme.getTypeValue(args[0]);
+	if (type != null)
+	  return type;
+      }
+    return Type.pointer_type;
   }
 }
