@@ -2,6 +2,7 @@ package kawa.standard;
 import kawa.lang.*;
 import gnu.mapping.*;
 import gnu.expr.*;
+import gnu.kawa.util.*;
 
 /**
  * The Syntax transformer that re-writes the "define" Scheme primitive.
@@ -16,6 +17,61 @@ public class define extends Syntax implements Printable
     this.makePrivate = makePrivate;
   }
 
+  public boolean scanForDefinitions (Pair st, java.util.Vector forms,
+                                     ScopeExp defs, Translator tr)
+  {
+    if (! (st.cdr instanceof Pair))
+      return super.scanForDefinitions(st, forms, defs, tr);
+    Pair p = (Pair) st.cdr;
+    Object name = p.car;
+    Pair name_pair = null;
+    String sym = null;
+    if (name instanceof String)
+      {
+	sym = (String) name;
+      }
+    else if (name instanceof Pair)
+      {
+        name_pair = (Pair) name;
+        if (name_pair.car instanceof String)
+	  sym = (String) name_pair.car;
+      }
+    if (sym != null)
+      {
+	Declaration decl = defs.lookup(sym);
+	if (decl == null)
+	  {
+	    decl = new Declaration(sym);
+	    defs.addDeclaration(decl);
+	  }
+	else
+	  tr.error('w', "duplicate declaration for `"+sym+"'");
+	Object declForm = name_pair == null ? (Object) decl
+	  : (Object) tr.makePair(name_pair, decl, name_pair.cdr);
+	p = tr.makePair(p, declForm, p.cdr);
+	st = tr.makePair(st, this, p);
+        if (defs instanceof ModuleExp)
+          {
+            if (! makePrivate)
+              {
+                decl.setCanRead(true);
+		// (define (f) ...) defaults f to being read-only,
+		// unless f is assigned to in this module.
+		if (name instanceof String || Compilation.usingTailCalls)
+		  decl.setCanWrite(true);
+              }
+          }
+        if (declForm instanceof PairWithPosition)
+          {
+            PairWithPosition declPos = (PairWithPosition) declForm;
+            decl.setFile(declPos.getFile());
+            decl.setLine(declPos.getLine(), declPos.getColumn());
+          }
+      }
+    forms.addElement (st);
+    return true;
+  }
+
   public Expression rewriteForm (Pair form, Translator tr)
   {
     Object obj = form.cdr;
@@ -29,7 +85,7 @@ public class define extends Syntax implements Printable
 	if (p1.car instanceof String && p1.cdr instanceof Pair)
 	  {
 	    Pair p2 = (Pair) p1.cdr;
-	    if (p2.cdr == List.Empty)
+	    if (p2.cdr == LList.Empty)
 	      {
 		name = (String) p1.car;
 		value = tr.rewrite (p2.car);
@@ -38,10 +94,10 @@ public class define extends Syntax implements Printable
 	else if (p1.car instanceof Declaration && p1.cdr instanceof Pair)
 	  {
 	    Pair p2 = (Pair) p1.cdr;
-	    if (p2.cdr == List.Empty)
+	    if (p2.cdr == LList.Empty)
 	      {
 		decl = (Declaration) p1.car;
-                name = decl.symbol();
+                name = decl.getName();
 		value = tr.rewrite (p2.car);
 	      }
 	  }
@@ -55,7 +111,7 @@ public class define extends Syntax implements Printable
             else if (p2.car instanceof Declaration)
               {
                 decl = (Declaration) p2.car;
-                name = decl.symbol();
+                name = decl.getName();
               }
             if (name != null)
               {
@@ -81,7 +137,10 @@ public class define extends Syntax implements Printable
     if (decl != null)
       {
 	sexp.binding = decl;
-	decl.noteValue (value);
+	if (decl.context instanceof ModuleExp && ! makePrivate
+	    && decl.getCanWrite())
+	  value = null;
+	decl.noteValue(value);
       }
     if (! (tr.currentScope() instanceof ModuleExp))
       {
@@ -91,18 +150,6 @@ public class define extends Syntax implements Printable
                      +tr.currentScope());
             return sexp;
           }
-
-        /*
-        // FIXME:  Remove following once define is handled like define-private.
-
-	Object binding = tr.current_decls.get (name);
-	// Hygenic macro expansion may bind a renamed (uninterned) symbol
-	// to the original symbol.
-	if (binding == null || binding instanceof String)
-	  return tr.syntaxError ("invalid use of define");
-	sexp.binding = (Declaration) binding;
-	sexp.binding.noteValue (value);
-        */
       }
     return sexp;
   }
