@@ -1,5 +1,4 @@
 package kawa.standard;
-import kawa.lang.*;
 import gnu.math.Numeric;
 import gnu.mapping.*;
 
@@ -8,6 +7,6 @@ public class exact_p extends Procedure1
    public Object apply1 (Object arg1)
    {
      boolean result = arg1 instanceof Numeric && ((Numeric)arg1).isExact ();
-     return Interpreter.boolObject (result);
+     return result ? Boolean.TRUE : Boolean.FALSE;
    }
 }
