@@ -1,5 +1,6 @@
 package kawa.standard;
 import kawa.lang.*;
+import gnu.kawa.util.*;
 import gnu.bytecode.*;
 import gnu.mapping.*;
 import gnu.expr.*;
@@ -20,7 +21,7 @@ public class letrec extends Syntax implements Printable
     Pair pair = (Pair) obj;
     Object bindings = pair.car;
     Object body = pair.cdr;
-    int decl_count = List.length (bindings);
+    int decl_count = LList.length (bindings);
     Expression[] inits = new Expression[decl_count];
     LetExp let = new LetExp (inits);
     Expression[] newbody = new Expression[decl_count+1];
@@ -44,10 +45,10 @@ public class letrec extends Syntax implements Printable
 	if ("::".equals(binding.car)) // && "::" is unbound FIXME
 	  {
 	    if (! (binding.cdr instanceof Pair)
-		|| (binding = (Pair) binding.cdr).cdr == List.Empty)
+		|| (binding = (Pair) binding.cdr).cdr == LList.Empty)
 	      return tr.syntaxError("missing type after `::' in let");
 	  }
-	if (binding.cdr == List.Empty)
+	if (binding.cdr == LList.Empty)
 	  {
 	    init = binding.car;
 	  }
@@ -64,10 +65,10 @@ public class letrec extends Syntax implements Printable
       }
     tr.push(let);
     i = 0;
-    for (Variable var = let.firstVar(); var != null; var = var.nextVar(), i++)
+    for (Declaration decl = let.firstDecl();
+         decl != null;  decl = decl.nextDecl(), i++)
       {
 	Expression exp = tr.rewrite(orig_inits[i]);
-	Declaration decl = (Declaration) var;
 	newbody[i] = new SetExp(decl, exp);
 	decl.noteValue (exp);				
       }
