@@ -1,6 +1,6 @@
 package kawa.standard;
-import kawa.lang.*;
 import gnu.math.*;
+import gnu.mapping.*;
 
 /**
  * Implement the Scheme standard function "-".
@@ -9,8 +9,17 @@ import gnu.math.*;
 
 public class minus_oper extends ProcedureN
 {
+  public static Object apply (Object arg1)
+  {
+    return ((Numeric) arg1).neg();
+  }
+
+  public static Object apply (Object arg1, Object arg2)
+  {
+    return ((Numeric) arg1).sub(arg2);
+  }
+
   public Object applyN (Object[] args)
-      throws WrongArguments, WrongType, GenericError, UnboundSymbol
   {
     Numeric result = (Numeric) args[0];
     if (args.length == 1)
