@@ -1,7 +1,7 @@
 package kawa.standard;
-import kawa.lang.*;
 import gnu.math.IntNum;
 import gnu.math.Numeric;
+import gnu.mapping.*;
 
 /**
  * Implement the Scheme standard function "*".
@@ -10,8 +10,12 @@ import gnu.math.Numeric;
 
 public class multiply_oper extends ProcedureN
 {
+  public static Object apply (Object arg1, Object arg2)
+  {
+    return ((Numeric) arg1).mul(arg2);
+  }
+
   public Object applyN (Object[] args)
-      throws WrongArguments, WrongType, GenericError, UnboundSymbol
   {
     int len = args.length;
     if (len == 0)
