@@ -2,6 +2,9 @@ package kawa.standard;
 import kawa.lang.*;
 import gnu.mapping.*;
 import gnu.expr.*;
+import gnu.kawa.util.*;
+import gnu.kawa.reflect.Invoke;
+import gnu.bytecode.ClassType;
 
 /**
  * The Syntax transformer that re-writes the "quasiquote" Scheme primitive.
@@ -24,7 +27,7 @@ public class quasiquote extends Syntax implements Printable
 	depth--;
 	Pair pair_cdr;
 	if (! (pair.cdr instanceof Pair)
-	    || (pair_cdr = (Pair) pair.cdr).cdr != List.Empty)
+	    || (pair_cdr = (Pair) pair.cdr).cdr != LList.Empty)
 	  return tr.syntaxError ("invalid used of " + pair.car +
 				     " in quasiquote template");
 	if (depth == 0)
@@ -41,14 +44,13 @@ public class quasiquote extends Syntax implements Printable
       {
 	Pair pair_car_cdr;
 	if (! (pair_car.cdr instanceof Pair)
-	    || (pair_car_cdr = (Pair) pair_car.cdr).cdr != List.Empty)
+	    || (pair_car_cdr = (Pair) pair_car.cdr).cdr != LList.Empty)
 	  return tr.syntaxError ("invalid used of " + pair_car.car +
 				     " in quasiquote template");
-	Procedure append = kawa.standard.append.appendProcedure;
 	Expression[] args = new Expression[2];
 	args[0] = tr.rewrite (pair_car_cdr.car);
 	args[1] = coerceExpression (expanded_cdr);
-	return new ApplyExp (new QuoteExp (append), args);
+	return Invoke.makeInvokeStatic(appendType, "append", args);
       }
     Object expanded_car = expand (pair.car, depth, tr);
     if (expanded_car == pair.car && expanded_cdr == pair.cdr)
@@ -58,11 +60,10 @@ public class quasiquote extends Syntax implements Printable
       return new Pair (expanded_car, expanded_cdr);
     else
       {
-	Expression cons = new QuoteExp (kawa.standard.cons.consProcedure);
 	Expression[] args = new Expression[2];
 	args[0] = coerceExpression (expanded_car);
 	args[1] = coerceExpression (expanded_cdr);
-	return new ApplyExp (cons, args);
+	return Invoke.makeInvokeStatic(consType, "cons", args);
       }
   }
 
@@ -77,9 +78,9 @@ public class quasiquote extends Syntax implements Printable
   {
     if (template instanceof Pair)
       return expand_pair ((Pair) template, depth, tr);
-    else if (template instanceof kawa.lang.Vector)
+    else if (template instanceof FVector)
       {
-	kawa.lang.Vector vector = (kawa.lang.Vector) template;
+	FVector vector = (FVector) template;
 	int n = vector.length ();
 	Object[] buffer = new Object[n];
 	// For each element, the state is one of these four:
@@ -101,7 +102,7 @@ public class quasiquote extends Syntax implements Printable
 	      {
 		Pair pair_cdr;
 		if (! (pair.cdr instanceof Pair)
-		    || (pair_cdr = (Pair) pair.cdr).cdr != List.Empty)
+		    || (pair_cdr = (Pair) pair.cdr).cdr != LList.Empty)
 		  return tr.syntaxError ("invalid used of " + pair.car +
 					     " in quasiquote template");
 		buffer[i] = tr.rewrite (pair_cdr.car);
@@ -123,7 +124,7 @@ public class quasiquote extends Syntax implements Printable
 	if (max_state == 0)
 	  return vector;
 	if (max_state == 1)
-	  return new Vector (buffer);
+	  return new FVector (buffer);
 	Expression[] args = new Expression[n];
 	for (int i = 0;  i < n;  i++)
 	  {
@@ -135,22 +136,19 @@ public class quasiquote extends Syntax implements Printable
 	      {
 		Object[] arg1 = new Object[1];
 		arg1[0] = buffer[i];
-		args[i] = new QuoteExp (new Vector (arg1));
+		args[i] = new QuoteExp (new FVector (arg1));
 	      }
 	    else
 	      {
 		Expression[] arg1 = new Expression[1];
 		arg1[0] = (Expression) buffer[i];
-		Procedure func = kawa.standard.vector_v.vectorProcedure;
-		args[i] = new ApplyExp (new QuoteExp (func), arg1);
+		args[i] = Invoke.makeInvokeStatic(vectorType, "vector", arg1);
 	      }
 	  }
-	Procedure func;
 	if (max_state < 3)
-	  func = kawa.standard.vector_v.vectorProcedure;
+	  return Invoke.makeInvokeStatic(vectorType, "vector", args);
 	else
-	  func = kawa.standard.vector_append.vappendProcedure;
-	return new ApplyExp (new QuoteExp (func), args);
+	  return Invoke.makeInvokeStatic(vectorAppendType, "apply", args);
       }
     else
       return template;
@@ -160,8 +158,14 @@ public class quasiquote extends Syntax implements Printable
   {
     Pair pair;
     if (! (obj instanceof Pair)
-	|| (pair = (Pair) obj).cdr != List.Empty)
+	|| (pair = (Pair) obj).cdr != LList.Empty)
       return tr.syntaxError ("wrong number of arguments to quasiquote");
     return coerceExpression (expand (pair.car, 1, tr));
   }
+
+  static final ClassType consType = ClassType.make("kawa.lib.lists");
+  static final ClassType appendType = ClassType.make("kawa.standard.append");
+  static final ClassType vectorType = ClassType.make("kawa.lib.vectors");
+  static final ClassType vectorAppendType
+    = ClassType.make("kawa.standard.vector_append");
 }
