@@ -2,6 +2,7 @@ package kawa.standard;
 import kawa.lang.*;
 import gnu.mapping.*;
 import gnu.expr.*;
+import gnu.kawa.util.*;
 
 /**
  * The Syntax transformer that re-writes the Scheme "set!" primitive.
@@ -29,11 +30,11 @@ public class set_b extends Syntax implements Printable
 	Object args = pair.cdr;
 	/*
 	args = new Pair(match[1], args);
-	proc = new Pair("setter", new Pair(proc, List.Empty));
+	proc = new Pair("setter", new Pair(proc, LList.Empty));
 	return tr.rewrite(new Pair(proc, args));
 	*/
 
-	int nargs = List.length(args);
+	int nargs = LList.length(args);
 	Expression[] xargs = new Expression[nargs+1];
 	for (int i = 1; i <= nargs; i++)
 	  {
@@ -49,13 +50,13 @@ public class set_b extends Syntax implements Printable
       return tr.syntaxError ("first set! argument is not a variable name");
     String sym = (String) match[0];
     Expression value = tr.rewrite (match[1]);
-    Object binding = tr.current_decls.get (sym);
+    Object binding = tr.environ.get (sym);
     // Hygenic macro expansion may bind a renamed (uninterned) symbol
     // to the original symbol.  Here, use the original symbol.
     if (binding != null && binding instanceof String)
       return new SetExp ((String) binding, value);
     SetExp sexp = new SetExp (sym, value);
-    if (binding != null)
+    if (binding instanceof Declaration)
       {
 	Declaration decl = (Declaration) binding;
 	if (tr.isLexical(decl))
