@@ -1,5 +1,5 @@
 package kawa.standard;
-import kawa.lang.*;
+import gnu.kawa.util.*;
 import gnu.mapping.*;
 
 /**
@@ -9,7 +9,7 @@ import gnu.mapping.*;
 
 public class string_append extends ProcedureN
 {
-  public Object applyN (Object[] args)
+  public static FString stringAppend$V (Object[] args)
   {
     int count = args.length;
     java.lang.StringBuffer result = new java.lang.StringBuffer();
@@ -19,4 +19,9 @@ public class string_append extends ProcedureN
     }
     return new FString (result);
   }
+
+  public Object applyN (Object[] args)
+  {
+    return stringAppend$V(args);
+  }
 }
