@@ -1,5 +1,4 @@
 package kawa.standard;
-import kawa.lang.*;
 import gnu.mapping.Procedure2;
 
 /** Implement the standard Scheme procedure "string-ci=?". */
@@ -9,8 +8,8 @@ public class string_ci_equal_p extends Procedure2
    public Object apply2 (Object arg1, Object arg2)
    {
      if (arg1.toString().equalsIgnoreCase(arg2.toString()))
-       return Interpreter.trueObject;
+       return Boolean.TRUE;
      else
-       return Interpreter.falseObject;
+       return Boolean.FALSE;
    }
 }
