@@ -1,4 +1,4 @@
-(test-init "Miscellaneous" 66)
+(test-init "Miscellaneous" 85)
 
 ;;; DSSSL spec example 11
 (test '(3 4 5 6) (lambda x x) 3 4 5 6)
@@ -8,6 +8,16 @@
 	(list x y z i: i j: j))
       3 4 5 i: 6 i: 7)
 
+(test '(200 . 100)
+      (lambda (x #!optional (y (* 2 x)) (p (lambda () (cons y x))))
+	(p))
+      100)
+
+(test '(100 . 200)
+      (lambda (x #!optional (y (* 2 x)))
+	(cons x y))
+      100)
+
 (test #t keyword? foo:)
 (test #t keyword? 'foo:)
 (test #f keyword? 'foo\:)
@@ -21,6 +31,9 @@
 (define-unit ft 12in)
 (test 18in + 6in 1ft)
 
+(test 5 call-with-values (lambda () (values 4 5)) (lambda (a b) b))
+(test -1 call-with-values * -)
+
 ;;; This caused a spurious warning in earlier releases.
 (test '(1 2 3) 'let (let ((x (lambda l l))) (x 1 2 3)))
 
@@ -99,6 +112,20 @@
    (test #\Y read-char iport)
    (test #!eof read-char iport)))
 
+(define str-inport (open-input-string "(a . (b c . ())) 34"))
+(test #t input-port? str-inport)
+(test '(a b c) read str-inport)
+(test 34 read str-inport)
+(test #t eof-object? (peek-char str-inport))
+(close-input-port str-inport)
+
+(test "a(b c)" 'open-output-string
+      (let ((q (open-output-string))
+            (x '(a b c)))
+        (write (car x) q)
+        (write (cdr x) q)
+        (get-output-string q)))
+
 ;;; From: Hallvard Traetteberg <Hallvard.Traetteberg@idi.ntnu.no>
 ;;; Triggered bug with try-finally nested in an expression.
 
@@ -145,7 +172,9 @@
 		 (+ 100 *xx*)))
     *xx*))
 (define (twice-*xx*) (* 2 *xx*))
-(test '(220 . 3) 'fluid-test (let ((res (fluid-test 10))) (cons res *xx*)))
+(test '(220 . 3) 'fluid-let-1 (let ((res (fluid-test 10))) (cons res *xx*)))
+
+(test 'bar 'fluid-let-2 (fluid-let ((flt (lambda () 'bar))) (flt)))
 
 (section "closures")
 
@@ -254,6 +283,64 @@
     (lambda () (lambda () bar2))))
 (test 34 ((test-nesting-2)))
 
+(define (test-nesting-3 k l m n o)
+  (define (foo a b c d e f)
+    (list a b c d e f k l m n o))
+  (foo foo (+ k k) (+ k l) (+ k m) (+ k n) (+ k o)))
+(test '(20 21 22 23 24 10 11 12 13 14) 'test-nesting-3
+      (cdr (test-nesting-3 10 11 12 13 14)))
+
+;;; Testcase from  "Walter C. Pelissero" <wcp@lpds.sublink.org>:
+(test #t procedure?
+      (let* ((is-equal eqv?)
+             (false
+              (lambda ()
+                (is-equal 'bar 'foo)))
+             (foo (lambda () 'foo)))
+        (lambda ()
+          (foo))))
+
+(test #t pair?
+      (let* ((is-equal eqv?)
+             (false
+              (lambda ()
+                (is-equal 'bar 'foo)))
+             (foo (lambda () (false))))
+        (list
+         false
+         (lambda () (foo)))))
+
+(test #t pair?
+      (let* ((is-equal eqv?)
+             (false
+              (lambda ()
+                (is-equal 'bar 'foo)))
+             (foo (lambda () (false))))
+        (list
+         false
+         (lambda ()
+           (define (bar) (foo))
+           (list bar (bar))))))
+
+(test #t procedure?
+      (let () 
+        (define aa 20)
+        (define (foo) aa)
+        (define (bar)
+          (let loop ((arg 'bar))
+            (foo)
+            (not (loop (foo)))))
+        bar))
+
+(test #t not
+      (let* ((foo (lambda ()
+                    'foo))
+             (bar (lambda ()
+                    (let loop ((arg 'bar))
+                      (foo)
+                      (not (loop (foo)))))))
+        #f))
+
 (define (test-duplicate-names)
   (let ((bar #t)) (lambda () (lambda () bar)))
   (let ((bar #t)) (lambda () (lambda () bar)))
@@ -261,6 +348,27 @@
   97)
 (test 97 test-duplicate-names)
 
+(test #f 'mutual-recursion-1
+      (letrec ((a (lambda () (b)))
+               (b (lambda () (a))))
+        #f))
+(test #f 'mutual-recursion-2
+      (letrec ((a (lambda () 10))
+               (b (lambda () (a)))
+               (c (lambda () (e) (b)))
+               (d (lambda () (c)))
+               (e (lambda () (d))))
+        #f))
+
+;; Used to cause a verification error.
+(define (sql-rsmd-all op rsmd . iter)
+  (if (null? iter)
+      (sql-rsmd-all op rsmd (sql-rsmd-columncount rsmd) '())
+      (if (zero? (car iter))
+        (cadr iter)
+	  (sql-rsmd-all op rsmd (- (car iter) 1)
+			      (cons (op rsmd (car iter))
+					      (cadr iter))))))
 (define (test-location-local x)
   (let* ((xl (location x))  ;; test location of formal parameter x
 	 (z (xl))
@@ -268,3 +376,6 @@
     (set! (xl) (+ (zl) 100))
     x))
 (test 110 test-location-local 10)
+
+(test 15 'tail-call (let loop ((a 1) (b 2) (c 3) (d 4) (e 5) (f 6))
+  (if (> a 10) b (loop b c d e f (+ a b c)))))
